package cn.com.qmth.examcloud.core.oe.admin.api.controller;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamRecordDataBeanConvert;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamRecordDataDomain;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordDataSyncService;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import io.swagger.annotations.Api;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

@RestController
@Api(tags = "考试记录相关接口")
@RequestMapping("${$rmp.ctr.oe}/exam/record/data")
public class ExamRecordDataController {

    private static final Logger log = LoggerFactory.getLogger(ExamRecordDataController.class);

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private ExamRecordDataSyncService examRecordDataSyncService;

    @GetMapping("/findExamRecordDataEntity")
    public ExamRecordDataDomain findExamRecordDataEntity(@RequestParam Long examRecordDataId, @RequestParam(required = false) String fromCache) {
        if (fromCache != null) {
            examRecordDataId = examRecordDataSyncService.getExamRecordDataIdByCacheId(examRecordDataId);
            if (examRecordDataId == null) {
                log.warn("ExamRecordDataEntity not exist, id:{}", examRecordDataId);
                throw new StatusException("1001", "当前考试记录信息不存在");
            }
        }

        ExamRecordDataEntity examRecordDataEntity = GlobalHelper.getEntity(examRecordDataRepo, examRecordDataId, ExamRecordDataEntity.class);
        if (examRecordDataEntity == null) {
            log.warn("ExamRecordDataEntity not exist, id:{}", examRecordDataId);
            throw new StatusException("1001", "当前考试记录信息不存在");
        }

        ExamRecordDataDomain domain = ExamRecordDataBeanConvert.copyFrom(examRecordDataEntity);
        domain.setExamRecord(examRecordDataEntity);
        return domain;
    }

    @GetMapping("/findByExamStudentId")
    public List<ExamRecordDataDomain> findByExamStudentId(@RequestParam Long examStudentId) {
        List<ExamRecordDataEntity> examRecordEntities = examRecordDataRepo.findByExamStudentId(examStudentId);

        List<ExamRecordDataDomain> examRecordDataList = new ArrayList<>();
        if (CollectionUtils.isEmpty(examRecordEntities)) {
            return examRecordDataList;
        }

        for (ExamRecordDataEntity examRecordData : examRecordEntities) {
            ExamRecordDataDomain domain = ExamRecordDataBeanConvert.copyFrom(examRecordData);
            domain.setExamRecord(examRecordData);
            examRecordDataList.add(domain);
        }

        return examRecordDataList;
    }

}
