package cn.com.qmth.examcloud.core.oe.admin.api.controller;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordQuestionsRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamQuestionEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordQuestionsEntity;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordDataSyncService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordQuestionsService;
import cn.com.qmth.examcloud.marking.api.StudentPaperCloudService;
import cn.com.qmth.examcloud.marking.api.bean.SubjectiveScoreBean;
import cn.com.qmth.examcloud.marking.api.request.GetSubjectiveScoreReq;
import cn.com.qmth.examcloud.marking.api.response.GetSubjectiveScoreResp;
import cn.com.qmth.examcloud.question.commons.core.question.QuestionType;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * @author chenken
 * @date 2018年10月12日 上午9:48:47
 * @company QMTH
 * @description ExamRecordQuestionsController.java
 */
@Api(tags = "考试结束后-作答记录接口")
@RestController
@RequestMapping("${$rmp.ctr.oe}/examRecordQuestions")
public class ExamRecordQuestionsController extends ControllerSupport {

    private static final Logger log = LoggerFactory.getLogger(ExamRecordQuestionsController.class);

    @Autowired
    private ExamRecordQuestionsRepo examRecordQuestionsRepo;

    @Autowired
    private ExamRecordQuestionsService examRecordQuestionsService;

    @Autowired
    private ExamRecordDataSyncService examRecordDataSyncService;

    @Autowired
    private StudentPaperCloudService studentPaperCloudService;

    @ApiOperation(value = "获取交卷之后的所有试题作答信息")
    @GetMapping("/getExamRecordQuestions")
    public ExamRecordQuestionsEntity getExamRecordQuestions(
            @RequestParam Long examRecordDataId,
            @RequestParam(required = false) String fromCache,
            @RequestParam(required = false, defaultValue = "false") Boolean withScore) {
        Check.isNull(examRecordDataId, "examRecordDataId不能为空");
        if (fromCache != null) {
            examRecordDataId = examRecordDataSyncService.getExamRecordDataIdByCacheId(examRecordDataId);
            if (examRecordDataId == null) {
                throw new StatusException("1001", "未找到数据");
            }
        }

        ExamRecordQuestionsEntity result = examRecordQuestionsService.getExamRecordQuestionsAndFixExamRecordDataIfNecessary(examRecordDataId);
        if (withScore) {
            List<ExamQuestionEntity> examQuestions = result.getExamQuestionEntities();
            if (CollectionUtils.isEmpty(examQuestions)) {
                return result;
            }

            Map<Integer, Double> subjectiveScores = this.loadSubjectiveScores(examRecordDataId);
            for (ExamQuestionEntity examQuestion : examQuestions) {
                if (QuestionType.isObjective(examQuestion.getQuestionType())) {
                    // 客观题得分
                    if (examQuestion.getStudentAnswer() != null && examQuestion.getCorrectAnswer() != null
                            // && QuestionOptionHelper.isEqualAnswer(examQuestion.getCorrectAnswer(), examQuestion.getStudentAnswer())) {
                            && examQuestion.getStudentAnswer().equals(examQuestion.getCorrectAnswer())) {
                        examQuestion.setStudentScore(examQuestion.getQuestionScore());
                    } else {
                        examQuestion.setStudentScore(0d);
                    }
                } else {
                    // 主观题得分，为空时则未评分
                    Double score = subjectiveScores.get(examQuestion.getOrder());
                    examQuestion.setStudentScore(score);
                }
            }
        }

        return result;
    }

    private Map<Integer, Double> loadSubjectiveScores(Long examRecordDataId) {
        GetSubjectiveScoreReq req = new GetSubjectiveScoreReq();
        req.setExamRecordDataId(examRecordDataId);

        Map<Integer, Double> scoreMaps = null;
        try {
            GetSubjectiveScoreResp resp = studentPaperCloudService.subjectiveScore(req);
            List<SubjectiveScoreBean> scores = resp.getScores();
            if (CollectionUtils.isNotEmpty(scores)) {
                scoreMaps = scores.stream().collect(Collectors.toMap(SubjectiveScoreBean::getQuestionOrder, SubjectiveScoreBean::getScore, (k, v) -> v));
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }

        return scoreMaps != null ? scoreMaps : new HashMap<>();
    }

}
