package cn.com.qmth.examcloud.core.oe.admin.api.controller;

import cn.com.qmth.examcloud.api.commons.enums.AdminOperateType;
import cn.com.qmth.examcloud.api.commons.enums.DataRuleType;
import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.api.commons.security.bean.UserDataRule;
import cn.com.qmth.examcloud.core.oe.admin.service.AsyncExportService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamScoreService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.UserDataRules;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examscore.ExamScoreInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examscore.ExamScoreQuery;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examscore.ObjectiveScoreInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.util.AsyncExportConcurrentUtil;
import cn.com.qmth.examcloud.reports.commons.bean.AdminOperateReport;
import cn.com.qmth.examcloud.reports.commons.util.ReportsUtil;
import cn.com.qmth.examcloud.support.helper.IdentityNumberHelper;
import cn.com.qmth.examcloud.web.security.DataRule;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import java.util.HashSet;
import java.util.List;

/**
 * 考试分数相关接口
 *
 * @author: QMTH
 * @since: 2018/8/27
 */
@RestController
@Api(tags = "考试分数相关接口")
@RequestMapping("${$rmp.ctr.oe}/exam/score")
public class ExamScoreController extends ControllerSupport {

    @Autowired
    private AsyncExportService asyncExportService;

    @Autowired
    private ExamScoreService examScoreService;

    @DataRule(type = {DataRuleType.COURSE, DataRuleType.ORG})
    @PostMapping("/statistic/list")
    @ApiOperation(value = "分页查询成绩明细列表")
    public Page<ExamScoreInfo> scoreDetails(@RequestBody ExamScoreQuery query) {
        UserDataRules uds = new UserDataRules(getUserDataRule(DataRuleType.ORG), getUserDataRule(DataRuleType.COURSE));

        Page<ExamScoreInfo> examScoreList = examScoreService.getExamScoreList(uds, query);
        examScoreList.getContent().forEach(p -> {
            p.setIdentityNumber(IdentityNumberHelper.conceal(getRootOrgId(), p.getIdentityNumber()));
        });
        return examScoreList;
    }

    @PostMapping("/statistic/list/for/tool")
    @ApiOperation(value = "分页查询成绩明细列表（数据权限未限制，用于实施小工具）")
    public Page<ExamScoreInfo> scoreDetailsForTool(@RequestBody ExamScoreQuery query) {
        UserDataRule userDataRule = new UserDataRule();
        userDataRule.setGlobalStatus(true);
        userDataRule.setRefIds(new HashSet<>());
        UserDataRules uds = new UserDataRules(userDataRule, userDataRule);

        Page<ExamScoreInfo> examScoreList = examScoreService.getExamScoreList(uds, query);
        examScoreList.getContent().forEach(p -> {
            p.setIdentityNumber(IdentityNumberHelper.conceal(getRootOrgId(), p.getIdentityNumber()));
        });
        return examScoreList;
    }

    @GetMapping("/statistic/list/export/async")
    @ApiOperation(value = "导出“成绩统计”列表(异步)",
            notes = "参数示例：query={\"rootOrgId\":0,\"examId\":1,\"startLimit\":1,\"endLimit\":1}")
    public void exportExamScores(@RequestParam String query) {
        User user = getAccessUser();
        asyncExportService.exportExamScoreStatistics(query, user);
        AsyncExportConcurrentUtil.checkOrStartJob();
        ReportsUtil.report(new AdminOperateReport(user.getRootOrgId(), user.getUserId(), AdminOperateType.TYPE18.getDesc(), "导出条件:" + query));
    }

    @ApiOperation(value = "根据examStudentId获取客观分信息")
    @GetMapping("/queryObjectiveScoreList")
    public List<ObjectiveScoreInfo> queryObjectiveScoreList(@RequestParam Long examStudentId) {
        return examScoreService.queryObjectiveScoreList(examStudentId);
    }

    @ApiOperation(value = "按考生ID更新正确答案并重新算分（用于实施小工具）")
    @GetMapping("/updateCorrectAnswerAndReFixScore")
    public void updateCorrectAnswerAndReFixScore(
            @RequestParam Long examId,
            @RequestParam Long courseId,
            @RequestParam Long examStudentId) {
        examScoreService.updateCorrectAnswerAndReFixScore(examId, courseId, examStudentId);
    }

}
