/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-16 17:40:53.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.api.controller;

import cn.com.qmth.examcloud.api.commons.enums.AdminOperateType;
import cn.com.qmth.examcloud.api.commons.enums.DataRuleType;
import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.api.commons.security.bean.UserDataRule;
import cn.com.qmth.examcloud.commons.util.JsonMapper;
import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.excel.ExportUtils;
import cn.com.qmth.examcloud.core.oe.admin.dao.IllegallyTypeRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamStudentEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.IllegallyTypeEntity;
import cn.com.qmth.examcloud.core.oe.admin.service.AsyncExportService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamStudentService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.UserDataRules;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.*;
import cn.com.qmth.examcloud.core.oe.admin.service.util.AsyncExportConcurrentUtil;
import cn.com.qmth.examcloud.reports.commons.bean.AdminOperateReport;
import cn.com.qmth.examcloud.reports.commons.util.ReportsUtil;
import cn.com.qmth.examcloud.support.cache.bean.CourseCacheBean;
import cn.com.qmth.examcloud.support.helper.ExamCacheTransferHelper;
import cn.com.qmth.examcloud.support.helper.IdentityNumberHelper;
import cn.com.qmth.examcloud.web.security.DataRule;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 考生信息接口
 *
 * @author: QMTH
 * @since: 2018/8/16
 */
@RestController
@Api(tags = "考生信息相关接口")
@RequestMapping("${$rmp.ctr.oe}/exam/student")
public class ExamStudentController extends ControllerSupport {

    @Autowired
    private ExamStudentService examStudentService;

    @Autowired
    private AsyncExportService asyncExportService;

    @Autowired
    private IllegallyTypeRepo illegallyTypeRepo;

    /**
     * 查询课程本地缓存
     */
    private static Map<Long, CourseBean> courseBeanLocalCache;

    private static Timer timer;

    static {
        courseBeanLocalCache = new ConcurrentHashMap<Long, CourseBean>();
        timer = new Timer();
    }

    class CleanCourseBeanCacheTask extends TimerTask {

        @Override
        public void run() {
            courseBeanLocalCache.clear();
        }

    }

    public ExamStudentController() {
        long delay = 10 * 60 * 1000;//10分钟后开始
        long period = 30 * 60 * 1000;//30分钟清理一次
        timer.schedule(new CleanCourseBeanCacheTask(), delay, period);
    }

    @DataRule(type = {DataRuleType.COURSE, DataRuleType.ORG})
    @PostMapping("/examScheduling/list")
    @ApiOperation(value = "查询“考试进度详情”列表(分页)")
    public Page<ExamStudentInfo> getExamScheduling(@RequestBody ExamStudentQuery query) {
        Check.isNull(query, "查询参数不能为空！");
        Check.isNull(query.getExamId(), "考试批次不能为空！");
        UserDataRules uds = new UserDataRules(getUserDataRule(DataRuleType.ORG), getUserDataRule(DataRuleType.COURSE));
        Page<ExamStudentInfo> examStudentListPage = examStudentService.getExamStudentListPage(uds, query);

        examStudentListPage.getContent().forEach(p -> {
            p.setIdentityNumber(IdentityNumberHelper.conceal(p.getRootOrgId(), p.getIdentityNumber()));
        });
        return examStudentListPage;
    }

    @DataRule(type = {DataRuleType.COURSE, DataRuleType.ORG})
    @GetMapping("/examScheduling/list/export")
    @ApiOperation(value = "导出“考试进度详情”列表(Excel)", notes = "参数示例：query={\"pageNo\":1,\"pageSize\":10,\"examId\":123, ...}")
    public void exportExamStudentList(@RequestParam String query, HttpServletResponse response) throws Exception {
        ExamStudentQuery newQuery = new JsonMapper().parseJson(query, ExamStudentQuery.class);
        Check.isNull(newQuery, "请求参数不能为空！");
        Check.isNull(newQuery.getExamId(), "请先选择考试批次！");
        UserDataRules uds = new UserDataRules(getUserDataRule(DataRuleType.ORG), getUserDataRule(DataRuleType.COURSE));
        List<ExamStudentInfo> examStudentInfoList = examStudentService.getExamStudentInfoList(uds, newQuery);
        List<ExamStudentExcel> list = ExamStudentEntityConvert.ofExcel(examStudentInfoList);
        ExportUtils.exportEXCEL("考试详情列表", ExamStudentExcel.class, list, response);
        //        ExcelExportUtil.exportExcel("考试详情列表", ExamStudentExcel.class, list, response);
    }

    @GetMapping("/examScheduling/list/export/async")
    @ApiOperation(value = "导出“考试进度详情”列表(异步)", notes = "参数示例：query={\"pageNo\":1,\"pageSize\":10,\"examId\":123, ...}")
    public void exportExamSchedulingAsync(@RequestParam String query) {
        User user = getAccessUser();
        asyncExportService.exportExamScheduling(query, user);
        AsyncExportConcurrentUtil.checkOrStartJob();
        ReportsUtil.report(new AdminOperateReport(user.getRootOrgId(), user.getUserId(), AdminOperateType.TYPE19.getDesc(), "导出条件:" + query));
    }

    @DataRule(type = {DataRuleType.COURSE, DataRuleType.ORG})
    @PostMapping("/unfinished/list")
    @ApiOperation(value = "查询“缺考登记”列表(分页)")
    public Page<ExamStudentInfo> getExamStudentUnFinishedList(@RequestBody ExamStudentQuery query) {
        Check.isNull(query, "查询参数不能为空！");
        Check.isNull(query.getExamId(), "考试批次不能为空！");
        query.setFinished(0);
        UserDataRules uds = new UserDataRules(getUserDataRule(DataRuleType.ORG), getUserDataRule(DataRuleType.COURSE));
        Page<ExamStudentInfo> examStudentListPage = examStudentService.getExamStudentListPage(uds, query);
        examStudentListPage.getContent().forEach(p -> {
            p.setIdentityNumber(IdentityNumberHelper.conceal(p.getRootOrgId(), p.getIdentityNumber()));
        });
        return examStudentListPage;
    }

    @DataRule(type = {DataRuleType.COURSE, DataRuleType.ORG})
    @GetMapping("/unfinished/list/export")
    @ApiOperation(value = "导出“缺考登记”列表(Excel)", notes = "参数示例：query={\"pageNo\":1,\"pageSize\":10,\"examId\":123, ...}")
    public void exportExamStudentUnFinishedList(@RequestParam String query, HttpServletResponse response) throws Exception {
        ExamStudentQuery newQuery = new JsonMapper().parseJson(query, ExamStudentQuery.class);
        Check.isNull(newQuery, "请求参数不能为空！");
        Check.isNull(newQuery.getExamId(), "请先选择考试批次！");
        newQuery.setFinished(0);//未完成
        UserDataRules uds = new UserDataRules(getUserDataRule(DataRuleType.ORG), getUserDataRule(DataRuleType.COURSE));
        List<ExamStudentInfo> examStudentInfos = examStudentService.getExamStudentInfoList(uds, newQuery);
        List<ExamStudentUnFinishedExcel> list = ExamStudentEntityConvert.ofUnFinishedExcel(examStudentInfos);
        ExportUtils.exportEXCEL("缺考登记列表", ExamStudentUnFinishedExcel.class, list, response);
    }

    @DataRule(type = {DataRuleType.COURSE, DataRuleType.ORG})
    @PostMapping("/reexamine/list")
    @ApiOperation(value = "查询重考考生列表(分页)")
    public Page<ExamStudentInfo> getReExamineStudentList(@RequestBody ExamStudentQuery query) {
        Check.isNull(query, "查询参数不能为空！");
        Check.isNull(query.getExamId(), "考试批次不能为空！");
        UserDataRules uds = new UserDataRules(getUserDataRule(DataRuleType.ORG), getUserDataRule(DataRuleType.COURSE));
        Page<ExamStudentInfo> reExamineStudentList = examStudentService.getReExamineStudentList(uds, query);
        reExamineStudentList.getContent().forEach(p -> {
            p.setIdentityNumber(IdentityNumberHelper.conceal(p.getRootOrgId(), p.getIdentityNumber()));
        });
        return reExamineStudentList;
    }

    @PostMapping("/reexamine")
    @ApiOperation(value = "设置重考")
    public void setReexamine(@RequestParam Long examStudentId,
                             @RequestParam(required = false) String reexamineType,
                             @RequestParam(required = false) String reexamineDetail) {
        if (examStudentId == null) {
            return;
        }
        examStudentService.setReexamine(examStudentId, reexamineType, reexamineDetail);
        User user = getAccessUser();
        StringBuilder sb = new StringBuilder();
        sb.append("考生ID:" + examStudentId);
        if (StringUtils.isNotEmpty(reexamineType)) {
            IllegallyTypeEntity it = illegallyTypeRepo.findByRootOrgIdAndCode(-1L, reexamineType);
            if (it != null) {
                sb.append(" 重考理由:" + it.getName());
            }
        }
        if (StringUtils.isNotEmpty(reexamineDetail)) {
            sb.append(" 详情描述:" + reexamineDetail);
        }
        ReportsUtil.report(new AdminOperateReport(user.getRootOrgId(), user.getUserId(), AdminOperateType.TYPE20.getDesc(), sb.toString()));
    }

    @PostMapping("/statistic/by/finished")
    @ApiOperation(value = "考试概况-按考试完成进度统计")
    public ExamStudentFinishedStatistic getExamStudentStatisticByFinished(@RequestParam Long examId,
                                                                          @RequestParam(required = false) Long examStageId) {
        if (examId == null) {
            return null;
        }
        return examStudentService.getExamStudentStatisticByFinished(examId, examStageId);
    }

    @DataRule(type = {DataRuleType.ORG})
    @PostMapping("/statistic/by/org")
    @ApiOperation(value = "考试概况-按学习中心统计")
    public List<ExamStudentOrgStatistic> getExamStudentStatisticByOrg(@RequestParam Long examId,
                                                                      @RequestParam(required = false) Long examStageId,
                                                                      @RequestParam(required = false) Long orgId) {
        if (examId == null) {
            return null;
        }
        return examStudentService.getExamStudentStatisticByOrg(getUserDataRule(DataRuleType.ORG), examId, examStageId, orgId);
    }

    @DataRule(type = {DataRuleType.ORG})
    @GetMapping("/statistic/by/org/export")
    @ApiOperation(value = "导出“考试概况-按学习中心统计”列表(Excel)")
    public void exportExamStudentStatisticByOrg(@RequestParam Long examId,
                                                @RequestParam(required = false) Long examStageId,
                                                @RequestParam(required = false) Long orgId,
                                                HttpServletResponse response) throws Exception {
        Check.isNull(examId, "请选择考试批次！");
        List<ExamStudentOrgStatistic> examStudentStatisticByOrgList =
                examStudentService.getExamStudentStatisticByOrg(getUserDataRule(DataRuleType.ORG), examId, examStageId, orgId);
        List<OrgCompleteProgressExcel> resultList =
                ExamStudentEntityConvert.copyFromExamStudentOrgStatistic(examStudentStatisticByOrgList);
        ExportUtils.exportEXCEL("学习中心完成进度", OrgCompleteProgressExcel.class, resultList, response);
    }

    /**
     * 查询课程
     *
     * @param examId
     * @param orgId
     * @return
     */
    @DataRule(type = DataRuleType.COURSE)
    @GetMapping("/findCoursesByExamIdAndOrgId")
    @ApiOperation(value = "查询课程")
    public List<CourseCacheBean> findCoursesByExamIdAndOrgId(@RequestParam Long examId,
                                                             @RequestParam(required = false) Long examStageId,
                                                             @RequestParam(required = false) Long orgId) {
        if (examId == null) {
            return null;
        }
        List<Long> courseIdList = examStudentService.findCoursesFromExamStudent(examId, examStageId, orgId);
        UserDataRule ud = getUserDataRule(DataRuleType.COURSE);
        List<CourseCacheBean> courseBeanList = new ArrayList<CourseCacheBean>();
        if (ud.assertEmptyQueryResult()) {
            return courseBeanList;
        }
        for (Long courseId : courseIdList) {
            if (!ud.assertNeedQueryRefIds() || ud.getRefIds().contains(courseId)) {
                CourseCacheBean courseBean = ExamCacheTransferHelper.getCachedCourse(courseId);
                courseBeanList.add(courseBean);
            }
        }
        return courseBeanList;
    }

    /**
     * 课程完成进度
     *
     * @param examId
     * @param courseId
     * @return
     */
    @DataRule(type = DataRuleType.COURSE)
    @GetMapping("/courseProgress/list")
    @ApiOperation(value = "课程完成进度")
    public List<CourseProgressInfo> queryCourseProgressInfos(@RequestParam Long examId,
                                                             @RequestParam(required = false) Long examStageId,
                                                             @RequestParam(required = false) Long courseId,
                                                             @RequestParam(required = false) String orderColumn) {
        if (examId == null) {
            return null;
        }
        UserDataRule ud = getUserDataRule(DataRuleType.COURSE);
        List<CourseProgressInfo> courseProgressInfoList =
                examStudentService.queryCourseProgressInfos(ud, examId, examStageId, courseId, orderColumn);
        if (courseProgressInfoList != null && courseProgressInfoList.size() > 0) {
            for (CourseProgressInfo courseProgressInfo : courseProgressInfoList) {
                long key = courseProgressInfo.getCourseId();
                CourseCacheBean courseBean = ExamCacheTransferHelper.getCachedCourse(key);
                courseProgressInfo.setCourseCode(courseBean.getCode());
                courseProgressInfo.setCourseName(courseBean.getName());
            }
        }
        return courseProgressInfoList;
    }

    @DataRule(type = DataRuleType.COURSE)
    @GetMapping("/courseProgress/list/export")
    @ApiOperation(value = "导出“课程完成进度”列表(Excel)")
    public void exportCourseProgressInfos(@RequestParam Long examId,
                                          @RequestParam(required = false) Long examStageId,
                                          @RequestParam(required = false) Long courseId,
                                          @RequestParam(required = false) String orderColumn,
                                          HttpServletResponse response) throws Exception {
        Check.isNull(examId, "请选择考试批次！");
        UserDataRule ud = getUserDataRule(DataRuleType.COURSE);
        List<CourseProgressInfo> courseProgressInfoList =
                examStudentService.queryCourseProgressInfos(ud, examId, examStageId, courseId, orderColumn);
        if (courseProgressInfoList != null && courseProgressInfoList.size() > 0) {
            for (CourseProgressInfo courseProgressInfo : courseProgressInfoList) {
                long key = courseProgressInfo.getCourseId();
                CourseCacheBean courseBean = ExamCacheTransferHelper.getCachedCourse(key);
                courseProgressInfo.setCourseCode(courseBean.getCode());
                courseProgressInfo.setCourseName(courseBean.getName());
            }
        }
        List<CourseCompleteProgressExcel> resultList =
                ExamStudentEntityConvert.copyFromCourseProgressInfo(courseProgressInfoList);
        ExportUtils.exportEXCEL("课程完成进度", CourseCompleteProgressExcel.class, resultList, response);
    }

    @PostMapping("/simple/list")
    @ApiOperation(value = "分页查询网考考生列表（用于实施小工具）")
    public Page<ExamStudentEntity> getExamStudentSimpleList(@RequestBody ExamStudentQuery query) {
        return examStudentService.getExamStudentSimpleList(query);
    }

}
