package cn.com.qmth.examcloud.core.oe.admin.api.controller;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.FileUtil;
import cn.com.qmth.examcloud.core.oe.admin.base.Constants;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.service.OfflineExamService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.OfflineExamCourseInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.uploadfile.FileInfo;
import cn.com.qmth.examcloud.web.config.SystemProperties;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.codec.digest.DigestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;


/**
 * @author chenken
 * @date 2018年9月5日 下午3:33:26
 * @company QMTH
 * @description 离线考试Controller
 */
@Api(tags = "离线考试控制")
@RestController
@RequestMapping("${$rmp.ctr.oe}/offlineExam")
public class OfflineExamController extends ControllerSupport {

    private static final Logger log = LoggerFactory.getLogger(OfflineExamController.class);

    @Autowired
    private OfflineExamService offlineExamService;

    @Autowired
    private SystemProperties systemConfig;

    /**
     * 获取离线考试列表
     *
     * @return
     */
    @ApiOperation(value = "获取离线考试列表")
    @GetMapping("/getOfflineCourse")
    public List<OfflineExamCourseInfo> getOfflineCourse() {
        User user = getAccessUser();
        return offlineExamService.getOfflineCourse(user.getUserId());
    }

    /**
     * 开始考试
     *
     * @param examStudentId
     */
    @ApiOperation(value = "离线考试:开始考试")
    @GetMapping("/startOfflineExam")
    public void startOfflineExam(@RequestParam long examStudentId) {
        offlineExamService.startOfflineExam(examStudentId);
    }

    /**
     * 交卷
     */
    @ApiOperation(value = "离线考试:交卷")
    @PostMapping("/submitPaper")
    public void submitPaper(@RequestParam long examRecordDataId,
                            @RequestPart(value = "file", required = false) MultipartFile file) throws Exception {
        Check.isNull(file, "上传文件不能为空");
        User user = getAccessUser();

        String fileSuffix = FileUtil.getFileSuffix(file.getOriginalFilename());
        FileInfo fileInfo = new FileInfo();
        fileInfo.setOriginalFileName(file.getOriginalFilename());
        fileInfo.setFileBytes(file.getBytes());
        fileInfo.setFileSize(file.getSize());
        fileInfo.setFileSuffix(fileSuffix);

        offlineExamService.submitPaper(examRecordDataId, fileInfo, user.getUserId());
    }

    /**
     * 交卷
     *
     * @param fileArray        文件数组
     * @param examRecordDataId 考试记录id
     * @param fileMd5Array     文件摘要信息
     * @throws Exception
     */
    @ApiOperation(value = "离线考试:多文件交卷")
    @PostMapping("/batchSubmitPaper")
    public void batchSubmitPaper(@ApiParam(value = "考试记录id") @RequestParam Long examRecordDataId,
                                 @ApiParam(value = "文件类型:ZIP、PDF、IMAGE（单选、三选一）") @RequestParam String fileType,
                                 @ApiParam(value = "文件MD5信息") @RequestParam String[] fileMd5Array,
                                 @ApiParam(value = "文件数组") @RequestPart(required = false) MultipartFile[] fileArray) throws Exception {
        Check.isNull(fileArray, "上传文件不能为空");
        User user = getAccessUser();

        if (fileArray.length != fileMd5Array.length) {
            throw new StatusException("400401", "文件数量和文件MD5数量不匹配");
        }

        List<FileInfo> fileInfoList = new ArrayList<>();
        for (int i = 0; i < fileArray.length; i++) {
            MultipartFile file = fileArray[i];
            log.info("OfflineExam file {} - {}", i + 1, file.getOriginalFilename());

            if (file.getSize() > Constants.ANSWER_FILE_MAX_SIZE * Constants.M_SIZE) {
                throw new StatusException("400402", "文件大小不能超过" + Constants.ANSWER_FILE_MAX_SIZE + "M");
            }

            String realMD5 = DigestUtils.md5Hex(file.getInputStream());
            String tempMD5 = fileMd5Array[i];
            if (!realMD5.equals(tempMD5)) {
                log.warn("realMD5 = {}, tempMD5 = {}", realMD5, tempMD5);
                throw new StatusException("400403", "文件MD5验证失败");
            }

            String fileSuffix = FileUtil.getFileSuffix(file.getOriginalFilename());
            FileInfo fileInfo = new FileInfo();
            fileInfo.setOriginalFileName(file.getOriginalFilename());
            fileInfo.setFileBytes(file.getBytes());
            fileInfo.setFileSize(file.getSize());
            fileInfo.setFileSuffix(fileSuffix);
            fileInfo.setMd5(realMD5);
            fileInfoList.add(fileInfo);
        }

        offlineExamService.batchSubmitPaper(examRecordDataId, fileInfoList, fileType, user.getUserId());
    }

}
