package cn.com.qmth.examcloud.core.oe.admin.api.controller.client;

import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.FileUtil;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamRecordDataBeanConvert;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamRecordDataDomain;
import cn.com.qmth.examcloud.core.oe.admin.base.Constants;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordPaperStructEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordQuestionsEntity;
import cn.com.qmth.examcloud.core.oe.admin.service.*;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.*;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examscore.ObjectiveScoreInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.uploadfile.FileInfo;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.codec.digest.DigestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;

@Api(tags = "（客户端）考试过程相关接口")
@RestController
@RequestMapping("${$rmp.ctr.oe}/client/exam/process")
public class ExamProcessController extends ControllerSupport {

    private static final Logger log = LoggerFactory.getLogger(ExamProcessController.class);

    @Autowired
    private ExamStudentService examStudentService;

    @Autowired
    private PracticeService practiceService;

    @Autowired
    private ExamRecordDataSyncService examRecordDataSyncService;

    @Autowired
    private ExamRecordPaperStructService examRecordPaperStructService;

    @Autowired
    private ExamRecordQuestionsService examRecordQuestionsService;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private ExamScoreService examScoreService;

    @Autowired
    private OfflineExamService offlineExamService;

    @ApiOperation(value = "获取在线考试待考列表")
    @PostMapping("/queryExamList")
    public List<OnHandExamInfo> queryExamList() {
        User user = getAccessUser();
        return examStudentService.queryOnlineExamList(user.getUserId(), ExamType.ONLINE);
    }

    @ApiOperation(value = "获取在线考试已结束列表")
    @PostMapping("/queryExamEndList")
    public List<OnHandExamInfo> queryExamEndList() {
        User user = getAccessUser();
        return examStudentService.queryOnlineExamEndList(user.getUserId(), ExamType.ONLINE);
    }

    @ApiOperation(value = "获取在线作业待考列表")
    @PostMapping("/queryHomeworkList")
    public List<OnHandExamInfo> queryHomeworkList() {
        User user = getAccessUser();
        return examStudentService.queryOnlineExamList(user.getUserId(), ExamType.ONLINE_HOMEWORK);
    }

    @ApiOperation(value = "练习课程列表")
    @PostMapping("/queryPracticeCourseList")
    public List<PracticeCourseInfo> queryPracticeCourseList(@RequestParam Long examId) {
        User user = getAccessUser();
        return practiceService.queryPracticeCourseList(examId, user.getUserId());
    }

    @ApiOperation(value = "课程练习记录详情")
    @PostMapping("/queryPracticeRecordList")
    public List<PracticeRecordInfo> queryPracticeRecordList(@RequestParam Long examStudentId) {
        return practiceService.queryPracticeRecordList(examStudentId);
    }

    @ApiOperation(value = "单次练习答题情况统计")
    @PostMapping("/getPracticeDetailInfo")
    public PracticeDetailInfo getPracticeDetailInfo(@RequestParam Long examRecordDataId, @RequestParam(required = false) String fromCache) {
        if (fromCache != null) {
            examRecordDataId = examRecordDataSyncService.getExamRecordDataIdByCacheId(examRecordDataId);
            if (examRecordDataId == null) {
                throw new StatusException("1001", "未找到数据");
            }
        }
        return practiceService.getPracticeDetailInfo(examRecordDataId);
    }

    @ApiOperation(value = "获取考试记录试卷结构")
    @PostMapping("/getExamRecordPaperStruct")
    public ExamRecordPaperStructEntity getExamRecordPaperStruct(@RequestParam Long examRecordDataId, @RequestParam(required = false) String fromCache) {
        if (fromCache != null) {
            examRecordDataId = examRecordDataSyncService.getExamRecordDataIdByCacheId(examRecordDataId);
            if (examRecordDataId == null) {
                throw new StatusException("1001", "未找到数据");
            }
        }
        return examRecordPaperStructService.getExamRecordPaperStruct(examRecordDataId);
    }

    @ApiOperation(value = "获取交卷之后的所有试题作答信息")
    @PostMapping("/getExamRecordQuestions")
    public ExamRecordQuestionsEntity getExamRecordQuestions(@RequestParam Long examRecordDataId, @RequestParam(required = false) String fromCache) {
        if (fromCache != null) {
            examRecordDataId = examRecordDataSyncService.getExamRecordDataIdByCacheId(examRecordDataId);
            if (examRecordDataId == null) {
                throw new StatusException("1001", "未找到数据");
            }
        }
        return examRecordQuestionsService.getExamRecordQuestionsAndFixExamRecordDataIfNecessary(examRecordDataId);
    }

    @ApiOperation(value = "获取当前考试记录信息")
    @PostMapping("/findExamRecordDataEntity")
    public ExamRecordDataDomain findExamRecordDataEntity(@RequestParam Long examRecordDataId, @RequestParam(required = false) String fromCache) {
        if (fromCache != null) {
            examRecordDataId = examRecordDataSyncService.getExamRecordDataIdByCacheId(examRecordDataId);
            if (examRecordDataId == null) {
                log.warn("ExamRecordDataEntity not exist, id:{}", examRecordDataId);
                throw new StatusException("1001", "当前考试记录信息不存在");
            }
        }

        ExamRecordDataEntity examRecordDataEntity = GlobalHelper.getEntity(examRecordDataRepo, examRecordDataId, ExamRecordDataEntity.class);
        if (examRecordDataEntity == null) {
            log.warn("ExamRecordDataEntity not exist, id:{}", examRecordDataId);
            throw new StatusException("1001", "当前考试记录信息不存在");
        }

        ExamRecordDataDomain domain = ExamRecordDataBeanConvert.copyFrom(examRecordDataEntity);
        domain.setExamRecord(examRecordDataEntity);
        return domain;
    }

    @ApiOperation(value = "根据examStudentId获取客观分信息")
    @PostMapping("/queryObjectiveScoreList")
    public List<ObjectiveScoreInfo> queryObjectiveScoreList(@RequestParam Long examStudentId) {
        return examScoreService.queryObjectiveScoreList(examStudentId);
    }

    @ApiOperation(value = "获取离线考试列表")
    @PostMapping("/getOfflineCourse")
    public List<OfflineExamCourseInfo> getOfflineCourse() {
        User user = getAccessUser();
        return offlineExamService.getOfflineCourse(user.getUserId());
    }

    @ApiOperation(value = "离线考试:开始考试")
    @PostMapping("/startOfflineExam")
    public void startOfflineExam(@RequestParam long examStudentId) {
        offlineExamService.startOfflineExam(examStudentId);
    }

    @ApiOperation(value = "离线考试:交卷")
    @PostMapping("/submitPaper")
    public void submitPaper(@RequestParam Long examRecordDataId,
                            @RequestPart(value = "file", required = false) MultipartFile file) throws Exception {
        Check.isNull(file, "上传文件不能为空");
        User user = getAccessUser();

        String fileSuffix = FileUtil.getFileSuffix(file.getOriginalFilename());
        FileInfo fileInfo = new FileInfo();
        fileInfo.setOriginalFileName(file.getOriginalFilename());
        fileInfo.setFileBytes(file.getBytes());
        fileInfo.setFileSize(file.getSize());
        fileInfo.setFileSuffix(fileSuffix);

        offlineExamService.submitPaper(examRecordDataId, fileInfo, user.getUserId());
    }

    @ApiOperation(value = "离线考试:多文件交卷")
    @PostMapping("/batchSubmitPaper")
    public void batchSubmitPaper(@ApiParam(value = "考试记录id") @RequestParam Long examRecordDataId,
                                 @ApiParam(value = "文件类型:ZIP、PDF、IMAGE（单选、三选一）") @RequestParam String fileType,
                                 @ApiParam(value = "文件MD5信息") @RequestParam String[] fileMd5Array,
                                 @ApiParam(value = "文件数组") @RequestPart(required = false) MultipartFile[] fileArray) throws Exception {
        Check.isNull(fileArray, "上传文件不能为空");
        User user = getAccessUser();

        if (fileArray.length != fileMd5Array.length) {
            throw new StatusException("400401", "文件数量和文件MD5数量不匹配");
        }

        List<FileInfo> fileInfoList = new ArrayList<>();
        for (int i = 0; i < fileArray.length; i++) {
            MultipartFile file = fileArray[i];
            log.info("OfflineExam file {} - {}", i + 1, file.getOriginalFilename());

            if (file.getSize() > Constants.ANSWER_FILE_MAX_SIZE * Constants.M_SIZE) {
                throw new StatusException("400402", "文件大小不能超过" + Constants.ANSWER_FILE_MAX_SIZE + "M");
            }

            String realMD5 = DigestUtils.md5Hex(file.getInputStream());
            String tempMD5 = fileMd5Array[i];
            if (!realMD5.equals(tempMD5)) {
                log.warn("realMD5 = {}, tempMD5 = {}", realMD5, tempMD5);
                throw new StatusException("400403", "文件MD5验证失败");
            }

            String fileSuffix = FileUtil.getFileSuffix(file.getOriginalFilename());
            FileInfo fileInfo = new FileInfo();
            fileInfo.setOriginalFileName(file.getOriginalFilename());
            fileInfo.setFileBytes(file.getBytes());
            fileInfo.setFileSize(file.getSize());
            fileInfo.setFileSuffix(fileSuffix);
            fileInfo.setMd5(realMD5);
            fileInfoList.add(fileInfo);
        }

        offlineExamService.batchSubmitPaper(examRecordDataId, fileInfoList, fileType, user.getUserId());
    }

}
