package cn.com.qmth.examcloud.core.oe.admin.api.provider;

import cn.com.qmth.examcloud.commons.util.OKHttpUtil;
import cn.com.qmth.examcloud.core.oe.admin.api.ExamScoreNoticeQueueCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.request.AddExamScoreNoticeQueueReq;
import cn.com.qmth.examcloud.core.oe.admin.api.response.AddExamScoreNoticeQueueResp;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamScoreNoticeQueueRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamScoreNoticeQueueEntity;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamScoreNoticeQueueService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamScoreObtainQueueService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.NotifyUrlInfo;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.Date;
import java.util.List;

@Api(tags = "定时发送获取分数通知接口")
@RestController
@RequestMapping("${$rmp.cloud.oe}/examScoreNoticeQueue")
public class ExamScoreNoticeQueueCloudServiceProvider extends ControllerSupport implements ExamScoreNoticeQueueCloudService {

    private static final long serialVersionUID = -2622841462905285211L;

    @Autowired
    private ExamScoreNoticeQueueRepo examScoreNoticeQueueRepo;

    @Autowired
    private ExamScoreObtainQueueService examScoreObtainQueueService;

    @Autowired
    private ExamScoreNoticeQueueService examScoreNoticeQueueService;

    /**
     * 定时发送分数获取通知 定时任务每5分钟发送一次
     */
    @ApiOperation(value = "定时发送获取分数通知接口")
    @PostMapping("/sendObtainScoreNotice")
    @Override
    public void sendObtainScoreNotice() {
        // 获取所有的通知队列
        List<ExamScoreNoticeQueueEntity> examScoreNoticeQueueList = examScoreNoticeQueueRepo.findAll();
        if (CollectionUtils.isEmpty(examScoreNoticeQueueList)) {
            return;
        }

        for (ExamScoreNoticeQueueEntity noticeEntity : examScoreNoticeQueueList) {
            // 获取当前组织机构的通知对象
            NotifyUrlInfo notifyUrlInfo = examScoreObtainQueueService.getNotifyUrlInfo(noticeEntity.getRootOrgId());

            // 只有配置了通知接口的才发送通知
            if (StringUtils.isBlank(notifyUrlInfo.getNotifyUrl())) {
                // 未配置推送地址的则清理掉
                examScoreNoticeQueueRepo.deleteById(noticeEntity.getRootOrgId());
                LOGGER.warn("通知学校获取成绩，尚未配置学校的推送地址！rootOrgId:{}", noticeEntity.getRootOrgId());
                continue;
            }

            try {
                OKHttpUtil.call(notifyUrlInfo.getHttpMethod(), notifyUrlInfo.getNotifyUrl());

                noticeEntity.setFailTimes(0L);
                noticeEntity.setUpdateTime(new Date());
                examScoreNoticeQueueRepo.save(noticeEntity);

                LOGGER.warn("通知学校获取成绩成功！rootOrgId:{} notifyUrl:{}", noticeEntity.getRootOrgId(), notifyUrlInfo.getNotifyUrl());
            } catch (Exception e) {
                if (e instanceof UnknownHostException || e instanceof SocketException) {
                    // 如果是由于连接超时,或者读取数据超时导致异常,需要对发送通知失败次数进行累加
                    long failTimes = noticeEntity.getFailTimes() == null ? 0 : noticeEntity.getFailTimes();
                    noticeEntity.setFailTimes(failTimes + 1);
                    noticeEntity.setUpdateTime(new Date());
                    examScoreNoticeQueueRepo.save(noticeEntity);
                }
                LOGGER.error("通知学校获取成绩失败！rootOrgId:{} notifyUrl:{} err:{}",
                        noticeEntity.getRootOrgId(), notifyUrlInfo.getNotifyUrl(), e.getMessage());
            }
        }
    }

    /**
     * 添加成绩通知队列
     */
    @ApiOperation(value = "添加成绩通知队列")
    @PostMapping("/addExamScoreNoticeQueue")
    @Override
    public AddExamScoreNoticeQueueResp addExamScoreNoticeQueue(@RequestBody AddExamScoreNoticeQueueReq req) {
        //如果不存在则添加通知队列，否则不做任何处理
        examScoreNoticeQueueService.addExamScoreNoticeQueue(req.getRootOrgId());
        return new AddExamScoreNoticeQueueResp();
    }

}
