package cn.com.qmth.examcloud.core.oe.admin.api.provider;

import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamStudentBeanConvert;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamStudentRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamStudentEntity;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamStudentService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.ExamStudentInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.cache.ExamStudentCache;
import cn.com.qmth.examcloud.global.api.HandleSyncCloudService;
import cn.com.qmth.examcloud.global.api.request.*;
import cn.com.qmth.examcloud.global.api.response.*;
import cn.com.qmth.examcloud.support.CacheConstants;
import cn.com.qmth.examcloud.web.helpers.SequenceLockHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @author chenken
 * @date 2018年10月29日 下午4:39:24
 * @company QMTH
 * @description 数据同步相关接口
 */
@Api(tags = "数据同步相关接口")
@RestController
@RequestMapping("${$rmp.cloud.oe}/exam/dataSync")
public class SyncCloudServiceProvider extends ControllerSupport implements HandleSyncCloudService {

    /**
     *
     */
    private static final long serialVersionUID = -4093980356473146740L;

    @Autowired
    private ExamStudentService examStudentService;

    @Autowired
    private ExamStudentRepo examStudentRepo;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private ExamStudentCache examStudentCache;

    @ApiOperation(value = "同步考生")
    @PostMapping("/syncExamStudent")
    public SyncExamStudentResp syncExamStudent(@RequestBody SyncExamStudentReq req) {
        //此锁是为了保证同步考生和交卷后更新考生信息不冲突
        String sequenceLockKey = CacheConstants.LOCK_EXAM_CONTROL + req.getStudentId();

        try {
            //添加考试控制全局锁
            SequenceLockHelper.getLockSimple(sequenceLockKey);

            if ("delete".equals(req.getSyncType())) {
                ExamStudentEntity examStudent = examStudentRepo.findByExamStudentId(req.getId());
                if (examStudent != null && (examStudent.getFinished() == null || !examStudent.getFinished())) {
                    examStudentRepo.delete(examStudent);
                    examStudentCache.remove(examStudent.getId());
                }
            } else if ("update".equals(req.getSyncType())) {
                ExamStudentInfo examStudent = ExamStudentBeanConvert.of(req);
                List<Long> examStudentIdList = examStudentService.saveExamStudentList(Lists.newArrayList(examStudent));
                for (Long cur : examStudentIdList) {
                    examStudentCache.refresh(cur);
                }
            }

            SyncExamStudentResp resp = new SyncExamStudentResp();
            return resp;
        } finally {
            SequenceLockHelper.releaseLockSimple(sequenceLockKey);
        }
    }

    @ApiOperation(value = "同步课程")
    @PostMapping("/syncCourse")
    public SyncCourseResp syncCourse(@RequestBody SyncCourseReq req) {
        Long courseId = req.getId();
        String courseLevel = req.getLevel();
        examStudentRepo.updateCourse(courseId, courseLevel);
        examRecordDataRepo.updateCourse(courseId, courseLevel);
        return null;
    }

    @ApiOperation(value = "同步学生")
    @PostMapping("/syncStudent")
    public SyncStudentResp syncStudent(@RequestBody SyncStudentReq req) {
        //考生表和考试记录表
        long studentId = req.getId();
        String studentName = req.getName();

        //		examStudentRepo.updateStudentNameAndStudentCode(studentId, studentName, studentCode);
        //    	examRecordRepo.updateStudentNameAndStudentCode(studentId, studentName, studentCode);
        examStudentRepo.updateStudentName(studentId, studentName);
        examRecordDataRepo.updateStudentName(studentId, studentName);
        //如果有需要解绑的学号信息，需要对学号进行解绑
        if (null != req.getUnboundStudentCodeList() && !req.getUnboundStudentCodeList().isEmpty()) {
            examStudentRepo.updateExamStudentCode(req.getIdentityNumber(), req.getUnboundStudentCodeList());
        }
        return null;
    }

    @ApiOperation(value = "同步机构")
    @PostMapping("/syncOrg")
    public SyncOrgResp syncOrg(@RequestBody SyncOrgReq req) {
        return null;
    }

    @ApiOperation(value = "同步专业")
    @PostMapping("syncSpecialty")
    public SyncSpecialtyResp syncSpecialty(@RequestBody SyncSpecialtyReq req) {
        return null;
    }

    @ApiOperation(value = "同步考试")
    @PostMapping("syncExam")
    public SyncExamResp syncExam(@RequestBody SyncExamReq req) {
        return null;
    }

    @Override
    public SyncUserResp syncUser(SyncUserReq req) {
        // TODO Auto-generated method stub
        return null;
    }


}
