/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-17 15:21:52.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.base.jpa;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * 查询条件类
 *
 * @author: QMTH
 * @since: 2018/8/17
 */
public class Searcher implements Serializable {
    protected static final long serialVersionUID = -1L;
    /**
     * 属性名
     */
    private String fieldName;
    /**
     * 属性值
     */
    private Object value;
    /**
     * 操作的表达式
     */
    private Op op;

    private List<Searcher> searchers = new ArrayList<>();

    public <T> Searcher eq(String fieldName, T value) {
        searchers.add(new Searcher(fieldName, value, Op.EQ));
        return this;
    }

    public <T> Searcher notEq(String fieldName, T value) {
        searchers.add(new Searcher(fieldName, value, Op.NOT_EQ));
        return this;
    }

    public <T> Searcher gt(String fieldName, T value) {
        searchers.add(new Searcher(fieldName, value, Op.GT));
        return this;
    }

    public <T> Searcher gte(String fieldName, T value) {
        searchers.add(new Searcher(fieldName, value, Op.GTE));
        return this;
    }

    public <T> Searcher lt(String fieldName, T value) {
        searchers.add(new Searcher(fieldName, value, Op.LT));
        return this;
    }

    public <T> Searcher lte(String fieldName, T value) {
        searchers.add(new Searcher(fieldName, value, Op.LTE));
        return this;
    }

    public <T> Searcher like(String fieldName, T value) {
        searchers.add(new Searcher(fieldName, value, Op.LIKE));
        return this;
    }

    public <T> Searcher leftLike(String fieldName, T value) {
        searchers.add(new Searcher(fieldName, value, Op.L_LIKE));
        return this;
    }

    public <T> Searcher rightLike(String fieldName, T value) {
        searchers.add(new Searcher(fieldName, value, Op.R_LIKE));
        return this;
    }

    public Searcher in(String fieldName, Collection<?> values) {
        if (values == null || values.size() == 0) {
            throw new IllegalArgumentException("Values must be not empty.");
        }
        if (values.size() == 1) {
            searchers.add(new Searcher(fieldName, values.iterator().next(), Op.EQ));
        } else {
            searchers.add(new Searcher(fieldName, values.toArray(), Op.IN));
        }
        return this;
    }

    public Searcher notIn(String fieldName, Collection<?> values) {
        if (values == null || values.size() == 0) {
            throw new IllegalArgumentException("Values must be not empty.");
        }
        if (values.size() == 1) {
            searchers.add(new Searcher(fieldName, values.iterator().next(), Op.NOT_EQ));
        } else {
            searchers.add(new Searcher(fieldName, values.toArray(), Op.NOT_IN));
        }
        return this;
    }

    public Searcher isNull(String fieldName) {
        searchers.add(new Searcher(fieldName, null, Op.IS_NULL));
        return this;
    }

    public Searcher notNull(String fieldName) {
        searchers.add(new Searcher(fieldName, null, Op.NOT_NULL));
        return this;
    }

    public List<Searcher> build() {
        return searchers;
    }

    private Searcher(String fieldName, Object value, Op op) {
        if (fieldName == null || "".equals(fieldName.trim())) {
            throw new IllegalArgumentException("FieldName must be not empty.");
        }
        this.fieldName = fieldName.trim();
        this.value = value;
        this.op = op;
    }

    public Searcher() {

    }

    public String getFieldName() {
        return fieldName;
    }

    public Object getValue() {
        return value;
    }

    public Op getOp() {
        return op;
    }

}