/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-17 15:21:52.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.base.jpa;

import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;

import javax.persistence.criteria.*;
import java.util.ArrayList;
import java.util.List;

/**
 * JPA查询工具类
 *
 * @author: QMTH
 * @since: 2018/8/17
 */
public class SpecUtils {
    /**
     * 默认当前页数
     */
    public static final int DEFAULT_PAGE_NO = 1;
    /**
     * 默认每页条数
     */
    public static final int DEFAULT_PAGE_SIZE = 10;

    public static Pageable buildPageable(Integer pageNo, Integer pageSize) {
        return buildPageable(pageNo, pageSize, Sort.unsorted());
    }

    public static Pageable buildPageable(Integer pageNo, Integer pageSize, Sort sort) {
        if (pageNo == null || pageNo < 1) {
            pageNo = DEFAULT_PAGE_NO;
        }
        if (pageSize == null || pageSize < 1) {
            pageSize = DEFAULT_PAGE_SIZE;
        }
        return PageRequest.of (pageNo - 1, pageSize, sort);
    }

    public static <T> Specification<T> buildSearchers(final Class<T> clazz, final List<Searcher> searchers) {
        return buildSearchers(clazz, searchers, false);
    }

    public static <T> Specification<T> buildSearchers(final Class<T> clazz, final List<Searcher> searchers, final boolean isOR) {
        if (searchers == null || searchers.size() == 0) {
            return null;
        }
        return new Specification<T>() {
            @Override
            public Predicate toPredicate(Root<T> root, CriteriaQuery<?> query, CriteriaBuilder builder) {
                List<Predicate> predicates = new ArrayList<>();
                for (Searcher filter : searchers) {
                    if (filter.getFieldName() == null || "".equals(filter.getFieldName())) {
                        continue;
                    }
                    String[] names = filter.getFieldName().split("\\.");
                    Path expression = root.get(names[0]);
                    for (int i = 1; i < names.length; i++) {
                        expression = expression.get(names[i]);
                    }
                    switch (filter.getOp()) {
                        case EQ:
                            predicates.add(builder.equal(expression, filter.getValue()));
                            break;
                        case NOT_EQ:
                            predicates.add(builder.notEqual(expression, filter.getValue()));
                            break;
                        case GT:
                            predicates.add(builder.greaterThan(expression, (Comparable) filter.getValue()));
                            break;
                        case LT:
                            predicates.add(builder.lessThan(expression, (Comparable) filter.getValue()));
                            break;
                        case GTE:
                            predicates.add(builder.greaterThanOrEqualTo(expression, (Comparable) filter.getValue()));
                            break;
                        case LTE:
                            predicates.add(builder.lessThanOrEqualTo(expression, (Comparable) filter.getValue()));
                            break;
                        case LIKE:
                            predicates.add(builder.like(expression, "%" + filter.getValue() + "%"));
                            break;
                        case L_LIKE:
                            predicates.add(builder.like(expression, "%" + filter.getValue()));
                            break;
                        case R_LIKE:
                            predicates.add(builder.like(expression, filter.getValue() + "%"));
                            break;
                        case IS_NULL:
                            predicates.add(builder.isNull(expression));
                            break;
                        case NOT_NULL:
                            predicates.add(builder.isNotNull(expression));
                            break;
                        case IN:
                            predicates.add(expression.in((Object[]) filter.getValue()));
                            break;
                        case NOT_IN:
                            predicates.add(expression.in((Object[]) filter.getValue()).not());
                            break;
                    }
                }
                if (!predicates.isEmpty()) {
                    if (isOR) {
                        //将所有条件用 or 联合起来
                        return builder.or(predicates.toArray(new Predicate[predicates.size()]));
                    } else {
                        //将所有条件用 and 联合起来
                        return builder.and(predicates.toArray(new Predicate[predicates.size()]));
                    }
                }
                return builder.conjunction();
            }
        };
    }

    /*public static Specification andMerge(Specification target1, Specification target2) {
        return Specifications.where(target1).and(target2);
    }

    public static Specification orMerge(Specification target1, Specification target2) {
        return Specifications.where(target1).or(target2);
    }*/

}