package cn.com.qmth.examcloud.core.oe.admin.base.utils;

import org.apache.http.client.config.RequestConfig;
import org.apache.http.impl.client.CloseableHttpClient; 
import org.apache.http.impl.client.HttpClientBuilder; 
import org.apache.http.impl.client.HttpClients; 
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

/** 
* 连接池工具类 
*/ 
@Component
public class HttpPoolUtil {

	private static final Logger log = LoggerFactory.getLogger(HttpPoolUtil.class);
	
	public static final String UTF8 = "UTF-8"; 
	public static volatile boolean isClosed = false; 
	
	public static final int maxTotalPool = 200;
	public static final int MAX_TIMEOUT = 15000;
	public static final int RequestTimeout = 5000;
	
	private static RequestConfig requestConfig; 
	private static HttpClientBuilder httpClientBuilder; 
	private static PoolingHttpClientConnectionManager poolConnManager;
	
	static { 
		// 设置连接池 
		poolConnManager = new PoolingHttpClientConnectionManager(); 
		poolConnManager.setMaxTotal(maxTotalPool);//设置连接池大小 
		poolConnManager.setDefaultMaxPerRoute(maxTotalPool); 
		
		RequestConfig.Builder configBuilder = RequestConfig.custom(); 
		// 设置连接超时 
		configBuilder.setConnectTimeout(MAX_TIMEOUT); 
		// 设置读取超时 
		configBuilder.setSocketTimeout(MAX_TIMEOUT); 
		// 设置从连接池获取连接实例的超时 
		configBuilder.setConnectionRequestTimeout(RequestTimeout); 
		// 在提交请求之前 测试连接是否可用 
		//configBuilder.setStaleConnectionCheckEnabled(true); 
		requestConfig = configBuilder.build(); 
		// 
		httpClientBuilder = HttpClients.custom()
									   .setConnectionManager(poolConnManager)
									   .setDefaultRequestConfig(requestConfig); 
		System.out.println(">>>>>>>>>>> PoolingHttpClientConnectionManager初始化成功 >>>>>>>>>>>");
	}
	
	/** 
	* 获取HttpClient客户端 
	* @return httpClient 
	*/ 
	/*public static CloseableHttpClient getClient() { 
		CloseableHttpClient httpClient = HttpClients.custom() 
		.setConnectionManager(poolConnManager) 
		.setDefaultRequestConfig(requestConfig) 
		.build(); 
		if(null == httpClient){ 
			httpClient = HttpClients.createDefault(); 
		} 
		return httpClient; 
	}*/
	
	/** 
	* 从http连接池里获取客户端实例 
	* @return httpClient 
	*/ 
	public static CloseableHttpClient getHttpClient() { 
		CloseableHttpClient httpClient = httpClientBuilder.build(); 
		if(null == httpClient){ 
			log.info("---------HttpClients.createDefault()---------");
			httpClient = HttpClients.createDefault(); 
		} 
		return httpClient; 
	}
	
	/** 
	* 关闭连接池资源 
	*/ 
	public synchronized static void closePool() { 
		if( !isClosed ){ 
			isClosed = true; 
			poolConnManager.close(); 
		} 
	}
} 
