package cn.com.qmth.examcloud.core.oe.admin.base.utils.baiduFaceVerify;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URI;
import java.net.URL;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.SysPropertyCacheBean;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;

/**
 * 
 * @author  	chenken
 * @date    	2018年10月8日 上午9:43:04
 * @company 	QMTH
 * @description 百度活体检测工具
 */
public class BaiduFaceVerifyUtil {
	
	/**
     * 获取API访问token
     * 该token有一定的有效期，需要自行管理，当失效时需重新获取.
     * @param ak - 百度云官网获取的 API Key
     * @param sk - 百度云官网获取的 Securet Key
     * @return assess_token 示例：
     * "24.460da4889caad24cccdb1fea17221975.2592000.1491995545.282335-1234567"
     */
    public static String getAccessToken(String ak, String sk) {
        // 获取token地址
        String authHost = "https://aip.baidubce.com/oauth/2.0/token?";
        String getAccessTokenUrl = authHost
                // 1. grant_type为固定参数
                + "grant_type=client_credentials"
                // 2. 官网获取的 API Key
                + "&client_id=" + ak
                // 3. 官网获取的 Secret Key
                + "&client_secret=" + sk;
        try {
            URL realUrl = new URL(getAccessTokenUrl);
            // 打开和URL之间的连接
            HttpURLConnection connection = (HttpURLConnection) realUrl.openConnection();
            connection.setRequestMethod("GET");
            connection.connect();
            // 获取所有响应头字段
            /*Map<String, List<String>> map = connection.getHeaderFields();
            // 遍历所有的响应头字段
            for (String key : map.keySet()) {
                System.err.println(key + "--->" + map.get(key));
            }*/
            // 定义 BufferedReader输入流来读取URL的响应
            BufferedReader in = new BufferedReader(new InputStreamReader(connection.getInputStream()));
            String result = "";
            String line;
            while ((line = in.readLine()) != null) {
                result += line;
            }
            /**
             * 返回结果
             */
            return result;
        } catch (Exception e) {
            e.printStackTrace(System.err);
        }
        return null;
    }
	
    /**
     * 重要提示代码中所需工具类
     * FileUtil,Base64Util,HttpUtil,GsonUtils请从
     * https://ai.baidu.com/file/658A35ABAB2D404FBF903F64D47C1F72
     * https://ai.baidu.com/file/C8D81F3301E24D2892968F09AE1AD6E2
     * https://ai.baidu.com/file/544D677F5D4E4F17B4122FBD60DB82B3
     * https://ai.baidu.com/file/470B3ACCA3FE43788B5A963BF0B625F3
     * 下载
     * param 必须是json数组
     */
     public static String faceVerify(String accessToken,String imageUrl) {
         // 请求url
         try {
             String param = getFaceVerifyParam(imageUrl);
             return HttpUtil.post(getFaceVerifyUrl(), accessToken, "application/json", param);
         } catch (Exception e) {
             e.printStackTrace();
         }
         return null;
     }
     
     /**
      * 得到活体检测参数
      * @param imageUrl
      * @return
      */
     private static String getFaceVerifyParam(String imageUrl){
    	JsonObject jsonObject = new JsonObject();
     	jsonObject.addProperty("image",imageUrl);
     	jsonObject.addProperty("image_type","URL");
     	JsonArray jsonArray = new JsonArray();
     	jsonArray.add(jsonObject);
     	return jsonArray.toString();
     }
    
    /*public static void main(String[] args) {
    	String accessToken = getAccessToken();
    	String imageUrl = "https://ecs-test-static.qmth.com.cn/capture_photo/6/1537953767307.jpg";
    	System.out.println(accessToken);
	}*/

    /**
     * 获取百度活体检测url
     * @return
     */
    private static String getFaceVerifyUrl(){
        SysPropertyCacheBean sysProperty = CacheHelper.getSysProperty("baidu.faceLiveness.url");
        if (sysProperty.getHasValue()){
            Object value = sysProperty.getValue();
            if (null!=value){
                String url = value.toString();
                return url;
            }
        }
        throw new StatusException("300005","未找到百度活体检测的配置路径");
    }
}
