package cn.com.qmth.examcloud.core.oe.admin.base.utils.excel;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;

import com.esotericsoftware.reflectasm.ConstructorAccess;
import com.esotericsoftware.reflectasm.FieldAccess;

import cn.com.qmth.examcloud.support.excel.ColumnSetting;
import cn.com.qmth.examcloud.support.excel.ExcelError;
import cn.com.qmth.examcloud.support.excel.ExcelErrorType;

/**
 * 
 * @Description: 读取excel
 * @author ting.yin
 * @date 2016年8月19日
 */
public class ExcelReader extends ExcelUtils {

	public ExcelReader(Class<?> dataClass) {
		super(dataClass);
	}

	/**
	 * 
	 * @param inputStream
	 *            输入流
	 * @param handle
	 *            单个对象处理器
	 * @return 错误信息集合
	 */
	public List<ExcelError> reader(InputStream inputStream,
			ExcelReaderHandle handle) {
		List<ExcelError> excelErrors = new ArrayList<ExcelError>();
		try {
			Workbook wb = WorkbookFactory.create(inputStream);
			Sheet sheet = wb.getSheetAt(0);
			for (int i = 1; i <= sheet.getLastRowNum(); i++) {
				Row row = sheet.getRow(i);
				if (row == null) {
					return excelErrors;
				}
				Object dto = ConstructorAccess.get(getDataClass()).newInstance();
				FieldAccess access = FieldAccess.get(getDataClass());
				try {
					for (int j = 0; j < row.getLastCellNum(); j++) {
						Cell cell = row.getCell(j);
						Object obj = convert(cell);
						ColumnSetting columnSetting = this.getColumnSettings().get(j);
						access.set(dto ,columnSetting.getHeader(), obj);
					}
				}catch (Exception e) {
					ExcelError error2 = new ExcelError(ExcelErrorType.EXCEL_FORMAT_ERROR.getName());
					error2.setRow(i+1);
					excelErrors.add(error2);
					e.printStackTrace();
					break;
				}
				ExcelError error = handle.handle(dto);
				if (error != null) {
					error.setRow(i+1);
					excelErrors.add(error);
				}
			}

		} catch (Exception e) {
			ExcelError error2 = new ExcelError(ExcelErrorType.EXCEL_HYPERLINK_FORMAT_ERROR.getName());
			excelErrors.add(error2);
			e.printStackTrace();
		} finally {
			try {
				inputStream.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return excelErrors;
	}

	@SuppressWarnings("deprecation")
	private Object convert(Cell cell) {
		if(cell!=null){
			switch (cell.getCellType()) {
			case Cell.CELL_TYPE_STRING:
				return cell.getStringCellValue().toString();
			case Cell.CELL_TYPE_BOOLEAN:
				return String.valueOf(cell.getBooleanCellValue());
			case Cell.CELL_TYPE_NUMERIC:
				return Math.round(cell.getNumericCellValue());
			}
		}
		
		return null;
	}
}
