package cn.com.qmth.examcloud.core.oe.admin.dao;

import java.util.Date;
import java.util.List;

import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamFileAnswerTempEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.FileAnswerAcknowledgeStatus;

@Repository
public interface ExamFileAnswerTempRepo
		extends JpaRepository<ExamFileAnswerTempEntity, Long>, JpaSpecificationExecutor<ExamFileAnswerTempEntity> {
	public ExamFileAnswerTempEntity findByExamRecordDataIdAndQuestionOrderAndFilePath(Long examRecordDataId,
																					  Integer questionOrder, String filePath);

	/**
	 * 根据考试记录id和状态获取最新的文件上传地址集合
	 * 
	 * @param examRecordDataId
	 * @param status
	 * @return
	 */
	@Query(value="select *" +
			" from ec_oe_exam_file_answer_temp" +
			" where exam_record_data_id= ?1 and status= 'CONFIRMED'",nativeQuery = true)
	public List<ExamFileAnswerTempEntity> findByExamRecordDataIdAndStatus(Long examRecordDataId, FileAnswerAcknowledgeStatus status);
	@Modifying
	@Query(nativeQuery = true, value = "delete from ec_oe_exam_file_answer_temp where exam_record_data_id = ?1 and exam_student_id = ?2 and question_order = ?3 and file_path=?4")
	public void delete(Long examRecordDataId, Long examStudentId, Integer questionOrder, String filePath);

	@Query(value="select * from ec_oe_exam_file_answer_temp order by id limit 100 ",nativeQuery = true)
	List<ExamFileAnswerTempEntity> findLimitTempFileAnswers(int limit);

	@Transactional
	@Modifying
	@Query(nativeQuery = true, value = "delete from ec_oe_exam_file_answer_temp where creation_time <= ?1")
	int deleteByCreationTime(Date creationTime);
}
