package cn.com.qmth.examcloud.core.oe.admin.dao;

import java.util.Date;
import java.util.List;

import javax.transaction.Transactional;

import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamStudentEntity;


/**
 * 
 * @author  	chenken
 * @date    	2018年8月13日 上午11:19:06
 * @company 	QMTH
 * @description 考试信息Repo
 */
@Repository
public interface ExamStudentRepo extends JpaRepository<ExamStudentEntity, Long>, JpaSpecificationExecutor<ExamStudentEntity> {
	
	public List<ExamStudentEntity> findByStudentIdAndExamIdIn(Long studentId,List<Long> examId);
	
	public List<ExamStudentEntity> findByStudentIdAndEnableAndExamIdIn(Long studentId,Boolean enable,List<Long> examId);

	List<ExamStudentEntity> findByStudentIdAndEnable(Long studentId,Boolean enable);

	public List<ExamStudentEntity> findByStudentId(Long studentId);

	public ExamStudentEntity findByExamStudentId(Long examStudentId);
	ExamStudentEntity findByIdentityNumberAndExamIdAndCourseId(String identityNumber,Long examId,Long courseId);
	
	public List<ExamStudentEntity> findByExamIdAndIdentityNumberIn(Long examId,List<String> identityNumbes);
	
	/**
	 * @param examId
	 * @param studentCodes
	 * @return
	 */
	public List<ExamStudentEntity> findByExamIdAndStudentCodeIn(Long examId,List<String> studentCodes);
	
	public List<ExamStudentEntity> findByRootOrgIdAndIdentityNumberIn(Long rootOrgId,List<String> identityNumbes);
	
	@Transactional
	@Modifying
	@Query(nativeQuery = true, value = "update ec_oe_exam_student set finished = 1 where exam_student_id = ?1")
	public void updateExamStudentFinished(Long examStudentId);
	
	public List<ExamStudentEntity> findByExamIdAndStudentId(Long examId,Long studentId);
	
	public List<ExamStudentEntity> findByExamId(Long examId);

	Long countByExamId(Long examId);

	Long countByExamIdAndFinished(Long examId,Boolean finished);
	
	@Transactional
	@Modifying
	@Query(nativeQuery = true, value = "update ec_oe_exam_student set student_name = ?2,student_code = ?3 where student_id = ?1")
	public void updateStudentNameAndStudentCode(Long studentId,String studentName,String studentCode);

	@Transactional
	@Modifying
	@Query(nativeQuery = true, value = "update ec_oe_exam_student set student_name = ?2 where student_id = ?1")
	public void updateStudentName(Long studentId,String studentName);

	@Transactional
	@Modifying
	@Query(nativeQuery = true, value = "update ec_oe_exam_student set course_level = ?2 where course_id = ?1")
	public void updateCourse(Long courseId,String courseLevel);

	/**
	 * 解绑学号
	 * @param studentCodes 学号集合
	 * @param identityNumber 身份证号
	 */
	@Transactional
	@Modifying
	@Query(nativeQuery = true, value = "update ec_oe_exam_student set student_code=null where identity_number= :identityNumber and student_code in :studentCodes")
	public void updateExamStudentCode(@Param("identityNumber") String identityNumber,@Param("studentCodes") List<String> studentCodes);


	@Transactional
	@Modifying
	@Query(nativeQuery = true,value = "delete from ec_oe_exam_student  where exam_id = ?1")
	public void deleteByExamId(Long examId);


	@Query(value = "select *  from ec_oe_exam_student where exam_id=?1 and id>=?2 order by id limit ?3",nativeQuery = true)
	List<ExamStudentEntity> getLimitExamStudentList(Long examId, Long startId, Integer size);

	ExamStudentEntity findByStudentIdAndExamIdAndCourseId(Long studentId,Long examId,Long courseId);

	@Query(value = "select *  from ec_oe_exam_student where enable=1 and finished=1 and exam_id=?1 and course_code=?2 and exam_student_id>=?3 order by id limit ?4",nativeQuery = true)
	List<ExamStudentEntity> getLimitExamStudentList(Long examId, String courseCode,Long startId, Integer size);

}
