/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-31 17:24:19.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import cn.com.qmth.examcloud.core.oe.admin.dao.ExamCaptureCameraInfoRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamCaptureRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamScoreRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamSyncCaptureRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.*;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.AuditStatus;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamAuditService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamCaptureService;
import cn.com.qmth.examcloud.core.oe.admin.service.IllegallyTypeService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examcapture.ExamCaptureAuditInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examcapture.ExamCaptureInfo;
import cn.com.qmth.examcloud.support.Constants;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.CourseCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.ExamStudentCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.OrgCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.SysPropertyCacheBean;
import cn.com.qmth.examcloud.support.helper.ExamCacheTransferHelper;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONException;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * 考试抓拍检测相关接口
 *
 * @author: QMTH
 * @since: 2018/8/29
 */
@Service
public class ExamCaptureServiceImpl implements ExamCaptureService {

    @Autowired
    private ExamCaptureRepo examCaptureRepo;
    
    @Autowired
    private ExamCaptureCameraInfoRepo examCaptureCameraInfoRepo;
    
    @Autowired
    private ExamAuditService examAuditService;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private ExamSyncCaptureRepo examSyncCaptureRepo;

    @Autowired
    private IllegallyTypeService illegallyTypeService;

    @Autowired
    private ExamScoreRepo examScoreRepo;

    private static final Logger LOG = LoggerFactory.getLogger(ExamCaptureServiceImpl.class);

    @Override
    public ExamCaptureAuditInfo getExamCaptureAuditDetail(Long examRecordDataId) {
        if (examRecordDataId == null) {
            return null;
        }
        ExamRecordDataEntity recordData = GlobalHelper.getEntity(examRecordDataRepo, examRecordDataId, ExamRecordDataEntity.class);
        if (recordData == null) {
            return null;
        }

        ExamAuditEntity audit = examAuditService.getExamAuditByExamRecordDataId(examRecordDataId);
        ExamCaptureAuditInfo detail = new ExamCaptureAuditInfo();
        if (audit != null) {
            if (audit.getStatus() != null) {
                detail.setStatus(audit.getStatus().getName());
            } else {
                detail.setStatus(AuditStatus.UN_PASS.getName());
            }
            detail.setDisciplineDetail(audit.getDisciplineDetail());
            if (audit.getDisciplineType() != null) {
                IllegallyTypeEntity illegallyType =
                        illegallyTypeService.getIllegallyType(recordData.getRootOrgId(), audit.getDisciplineType());
                detail.setDisciplineType(illegallyType.getName());
            }
        }

        detail.setExamRecordDataId(examRecordDataId);
        detail.setStudentId(recordData.getStudentId());
        detail.setStudentCode(recordData.getStudentCode());
        detail.setStudentName(recordData.getStudentName());
        detail.setIdentityNumber(recordData.getIdentityNumber());

        CourseCacheBean courseBean = ExamCacheTransferHelper.getCachedCourse(recordData.getCourseId());

        detail.setCourseCode(courseBean.getCode());
        detail.setCourseName(courseBean.getName());

        detail.setIsWarn(recordData.getIsWarn());
        detail.setIsAudit(recordData.getIsAudit());
        detail.setIsIllegality(recordData.getIsIllegality());
        detail.setFaceSuccessCount(recordData.getFaceSuccessCount());
        detail.setFaceFailedCount(recordData.getFaceFailedCount());
        detail.setFaceStrangerCount(recordData.getFaceStrangerCount());
        detail.setFaceTotalCount(recordData.getFaceTotalCount());
        detail.setFaceSuccessPercent(recordData.getFaceSuccessPercent());
        detail.setFaceLandmarkVal(recordData.getFaceLandmarkVal());
        detail.setBaiduFaceLivenessSuccessPercent(recordData.getBaiduFaceLivenessSuccessPercent());
        if (recordData.getFaceVerifyResult() != null) {
            detail.setFaceVerifyResult(recordData.getFaceVerifyResult().name());
        }
        detail.setVirtualCameraNames(getVirtualCameraNames(examRecordDataId));

        //同步人脸比较时抓拍照片的url
        detail.setSyncCaptureFileUrl(getSyncCaptureFileUrl(examRecordDataId));

        //组织机构
        OrgCacheBean org = CacheHelper.getOrg(recordData.getOrgId());
        detail.setOrgName(org.getName());

        //考生
        ExamStudentCacheBean examStudent = CacheHelper.getExamStudent(recordData.getExamStudentId());
        detail.setGrade(examStudent.getGrade());

        //课程层次
        detail.setCourseLevel(recordData.getCourseLevel());

        //客观题总分
        ExamScoreEntity examScore = examScoreRepo.findByExamRecordDataId(recordData.getId());
        if (null != examScore) {
            detail.setObjectiveTotalScore(String.valueOf(examScore.getObjectiveScore()));
        }

        detail.setSwitchScreenCount(null == recordData.getSwitchScreenCount() ? 0 : recordData.getSwitchScreenCount());

        return detail;
    }

    //获取同步时抓拍的照片
    private String getSyncCaptureFileUrl(Long examRecordDataId) {
        ExamSyncCaptureEntity syncCapture = examSyncCaptureRepo.findByExamRecordDataId(examRecordDataId);
        if (null == syncCapture) {
            return null;
        }

        return syncCapture.getFileUrl();
    }

    /**
     * 获取虚拟摄像头名称
     *
     * @param examRecordDataId
     * @return
     */
    @Override
    public String getVirtualCameraNames(Long examRecordDataId) {
        Set<String> virtualCameraNames = new HashSet<String>();
        List<ExamCaptureCameraInfoEntity> examCaptureList = examCaptureCameraInfoRepo.findByExamRecordDataIdAndVirtualCamera(examRecordDataId,true);
        if (examCaptureList != null && examCaptureList.size() > 0) {
            for (ExamCaptureCameraInfoEntity examCapture : examCaptureList) {
                String cameraInfos = examCapture.getName();
                if (StringUtils.isNotBlank(cameraInfos)) {
                	virtualCameraNames.add(cameraInfos);
                }
            }
        }
        StringBuffer sb = new StringBuffer();
        for (String cameraName : virtualCameraNames) {
            sb.append(cameraName).append(";");
        }
        return sb.toString();
    }


    @Override
    public List<ExamCaptureInfo> getExamCaptureList(Long examRecordDataId) {
        if (examRecordDataId == null) {
            return Lists.newArrayList();
        }
        List<ExamCaptureEntity> examCaptureEntityList = examCaptureRepo.findByExamRecordDataId(examRecordDataId);
        List<ExamCaptureInfo> examCaptureInfoList = new ArrayList<ExamCaptureInfo>();
        for (ExamCaptureEntity examCapture : examCaptureEntityList) {
            examCaptureInfoList.add(buildExamCaptureInfo(examCapture));
        }
        return examCaptureInfoList;
    }


    public ExamCaptureInfo buildExamCaptureInfo(ExamCaptureEntity entity) {
        ExamCaptureInfo info = new ExamCaptureInfo();
        info.setId(entity.getId());
        info.setExamRecordDataId(entity.getExamRecordDataId());
        info.setFileUrl(entity.getFileUrl());
        info.setReturnMessage(entity.getFaceCompareResult());
        info.setPass(entity.getIsPass());
        info.setStranger(entity.getIsStranger());

        String faceLivenessResult = entity.getFacelivenessResult();
        info.setIsFacelivenessPass(isFaceLivenessPass(faceLivenessResult));

//        String cameraInfos = entity.getCameraInfos();
//        info.setVirtualCameraNames(getVirtualCameraNames(cameraInfos));
        return info;
    }

//    private String getVirtualCameraNames(String cameraInfos) {
//        StringBuffer sb = new StringBuffer();
//        if (StringUtils.isNotBlank(cameraInfos)) {
//            JSONArray jsonArray;
//            try {
//                jsonArray = new JSONArray(cameraInfos);
//                for (int i = 0; i < jsonArray.length(); i++) {
//                    try {
//                        JSONObject jsonObject = (JSONObject) jsonArray.get(i);
//                        if (jsonObject.isNull("pid") || StringUtils.isBlank(jsonObject.getString("pid"))) {
//                            sb.append(jsonObject.getString("name") + ",");
//                        }
//                    } catch (JSONException e) {
//                        //主要针对json数组最后的空对象处理，不影响业务eg:"[   {      \"name\" : \"Camera\",      \"pid\" : \"\",      \"vid\" : \"\"   },]";
//                        LOG.error("抓拍照片格式不正确", e);
//                        continue;
//                    }
//                }
//            } catch (JSONException e) {
//                LOG.error("获取虚拟摄像头名称失败", e);
//                throw new StatusException("ExamCaptureService-002", "获取虚拟摄像头名称失败");
//            }
//
//            if (sb.lastIndexOf(",") > -1) {
//                return sb.substring(0, sb.lastIndexOf(","));
//            }
//        }
//        return sb.toString();
//    }

    private boolean isFaceLivenessPass(String faceLivenessResult) {
        if (StringUtils.isNotBlank(faceLivenessResult)) {
            // 兼容C端学生端情况，人脸真实性值只存在0或1两种值
            if ("0".equals(faceLivenessResult)) {
                return false;
            }
            if ("1".equals(faceLivenessResult)) {
                return true;
            }

            JSONObject jsonObject;
            try {
                jsonObject = new JSONObject(faceLivenessResult);
                if (jsonObject.has("error_code") && jsonObject.getInt("error_code") == 0 && jsonObject.has("result")) {
                    JSONObject resultJson = jsonObject.getJSONObject("result");
                    if (resultJson.has("face_liveness")) {
                        double faceLivenessVal = resultJson.getDouble("face_liveness");

                        Double baiduFacelivenessThreshold;
                        SysPropertyCacheBean baiduFacelivenessThresholdProperty = CacheHelper.getSysProperty("$baidu.faceliveness.threshold");
                        if (!baiduFacelivenessThresholdProperty.getHasValue()) {
                            baiduFacelivenessThreshold = Constants.DEFAULT_BAIDU_FACELIVENESS_THRESHOLD;
                        } else {
                            baiduFacelivenessThreshold = Double.valueOf(baiduFacelivenessThresholdProperty.getValue().toString());
                        }

                        if (faceLivenessVal > baiduFacelivenessThreshold) {
                            return true;
                        }
                    }
                }
            } catch (JSONException e) {
                LOG.error(e.getMessage(), e);
                return false;
            }
        }
        return false;
    }

}
