package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.core.oe.admin.api.request.CheckPaperInExamReq;
import cn.com.qmth.examcloud.core.oe.admin.api.response.CheckPaperInExamResp;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordDataService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.ExamStudentBean;
import cn.com.qmth.examcloud.support.cache.bean.CourseCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.ExamSettingsCacheBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * @author chenken
 * @date 2018/8/15 11:16
 * @company QMTH
 * @description 考试记录数据服务实现
 */
@Service("examRecordDataService")
public class ExamRecordDataServiceImpl implements ExamRecordDataService {

    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Override
    public ExamRecordDataEntity createOfflineExamRecordData(
            ExamStudentBean examStudent,
            ExamSettingsCacheBean examBean,
            CourseCacheBean courseBean,
            String basePaperId,
            Boolean fullyObjective,
            boolean randomPaper,
            double paperScore) {
        ExamRecordDataEntity examRecordDataEntity = new ExamRecordDataEntity();
        examRecordDataEntity.setExamId(examBean.getId());
        examRecordDataEntity.setExamType(ExamType.valueOf(examBean.getExamType()));

        examRecordDataEntity.setExamStudentId(examStudent.getExamStudentId());
        examRecordDataEntity.setStudentId(examStudent.getStudentId());
        examRecordDataEntity.setStudentCode(examStudent.getStudentCode());
        examRecordDataEntity.setStudentName(examStudent.getStudentName());
        examRecordDataEntity.setIdentityNumber(examStudent.getIdentityNumber());
        examRecordDataEntity.setOrgId(examStudent.getOrgId());
        examRecordDataEntity.setRootOrgId(examStudent.getRootOrgId());

        examRecordDataEntity.setCourseId(courseBean.getId());
        examRecordDataEntity.setCourseLevel(examStudent.getCourseLevel());

        // 采用千人千卷规则时，ID实际为randomPaperId
        examRecordDataEntity.setBasePaperId(basePaperId);
        // 是否采用千人千卷规则
        examRecordDataEntity.setRandomPaper(randomPaper);
        // 试卷总分
        examRecordDataEntity.setPaperScore(paperScore);

        examRecordDataEntity.setPaperType(examStudent.getPaperType());
        // examRecordDataEntity.setPaperStructId(paperStructId);

        examRecordDataEntity.setInfoCollector(examStudent.getInfoCollector());
        examRecordDataEntity.setStartTime(new Date());
        examRecordDataEntity.setExamRecordStatus(ExamRecordStatus.EXAM_ING);
        examRecordDataEntity.setExamOrder(1);
        examRecordDataEntity.setIsAllObjectivePaper(fullyObjective);
        examRecordDataEntity.setIsWarn(false);
        examRecordDataEntity.setIsAudit(false);
        examRecordDataEntity.setIsIllegality(false);
        return examRecordDataRepo.save(examRecordDataEntity);
    }

    @Override
    public Long getGetAwaitingAuditCount(Long rootOrgId, Long examId) {
        StringBuilder sql = new StringBuilder()
                .append(" select count(1) from ec_oe_exam_record_data t")
                .append(" where t.exam_id = ").append(examId)
                .append(" and t.exam_record_status != 'EXAM_INVALID'")
                .append(" and t.is_warn=1 and t.is_audit = 0")
                .append(" and t.root_org_id = ").append(rootOrgId);
        return jdbcTemplate.queryForObject(sql.toString(), Long.class);
    }

    @Override
    public CheckPaperInExamResp checkPaperInExam(CheckPaperInExamReq req) {
        Long id = examRecordDataRepo.getRecordIdByPaperId(req.getBasePaperId());
        CheckPaperInExamResp res = new CheckPaperInExamResp();
        if (id == null) {
            res.setInExam(false);
        } else {
            res.setInExam(true);
        }
        return res;
    }

}
