package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import cn.com.qmth.examcloud.commons.util.HttpMethod;
import cn.com.qmth.examcloud.commons.util.OKHttpUtil;
import cn.com.qmth.examcloud.core.oe.admin.base.Constants;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamScoreObtainQueueRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamScoreObtainQueueEntity;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamScoreObtainQueueService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.NotifyUrlInfo;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.SysPropertyCacheBean;
import cn.com.qmth.examcloud.support.enums.ExamProperties;
import cn.com.qmth.examcloud.support.helper.ExamCacheTransferHelper;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.util.Date;

/**
 * @author chenken
 * @date 2018年11月27日 下午2:05:32
 * @company QMTH
 * @description 成绩获取队列
 */
@Service("examScoreObtainQueueService")
public class ExamScoreObtainQueueServiceImpl implements ExamScoreObtainQueueService {

    private static final Logger log = LoggerFactory.getLogger(ExamScoreObtainQueueServiceImpl.class);

    @Autowired
    private ExamScoreObtainQueueRepo examScoreObtainQueueRepo;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Override
    public void saveExamScoreObtainQueue(Long examRecordDataId) {
        ExamRecordDataEntity examRecordData =
                GlobalHelper.getEntity(examRecordDataRepo, examRecordDataId, ExamRecordDataEntity.class);

        //是否推送分数
        String isPushScore = ExamCacheTransferHelper.getCachedExamProperty(examRecordData.getExamId(),
                examRecordData.getStudentId(), ExamProperties.PUSH_SCORE.name()).getValue();

        if (Constants.isTrue.equals(isPushScore)) {
            NotifyUrlInfo notifyUrlInfo = getNotifyUrlInfo(examRecordData.getRootOrgId());
            if (StringUtils.isNotBlank(notifyUrlInfo.getNotifyUrl())) {
                //保存队列
                ExamScoreObtainQueueEntity examScoreObtainQueue = examScoreObtainQueueRepo.findByExamRecordDataId(examRecordData.getId());
                if (examScoreObtainQueue == null) {
                    examScoreObtainQueue = new ExamScoreObtainQueueEntity();
                    examScoreObtainQueue.setCreationTime(new Date());
                }
                examScoreObtainQueue.setExamId(examRecordData.getExamId());
                examScoreObtainQueue.setExamRecordDataId(examRecordData.getId());
                examScoreObtainQueue.setRootOrgId(examRecordData.getRootOrgId());
                examScoreObtainQueue.setIsValid(true);
                examScoreObtainQueue.setUpdateTime(new Date());
                examScoreObtainQueueRepo.save(examScoreObtainQueue);
            }
        }
    }

    @Override
    public void sendObtainScoreNotify(Long rootOrgId) {
        if (rootOrgId == null) {
            return;
        }
        NotifyUrlInfo notifyUrlInfo = getNotifyUrlInfo(rootOrgId);

        if (StringUtils.isNotBlank(notifyUrlInfo.getNotifyUrl())) {
            try {
                OKHttpUtil.call(notifyUrlInfo.getHttpMethod(), notifyUrlInfo.getNotifyUrl());
                log.warn("通知学校获取成绩成功！rootOrgId:{} notifyUrl:{}", rootOrgId, notifyUrlInfo.getNotifyUrl());
            } catch (Exception e) {
                log.error("通知学校获取成绩失败！rootOrgId:{} notifyUrl:{} err:{}",
                        rootOrgId, notifyUrlInfo.getNotifyUrl(), e.getMessage());

            }
        }
    }

    /**
     * 根据组织机构获取通知路径相关信息
     *
     * @param rootOrgId 组织机构id
     * @return NotifyUrlInfo
     */
    @NotNull
    @Override
    public NotifyUrlInfo getNotifyUrlInfo(Long rootOrgId) {
        NotifyUrlInfo notifyUrlInfo = new NotifyUrlInfo();
        if (rootOrgId == null) {
            return notifyUrlInfo;
        }

        String notifyUrlKey = Constants.OE_EXAM_SCORE_NOTIFY_URL_PREFIX + rootOrgId;
        SysPropertyCacheBean notifyUrlKeyProperty = CacheHelper.getSysProperty(notifyUrlKey);

        String notifyUrlMethodKey = Constants.OE_EXAM_SCORE_NOTIFY_URL_HTTP_METHOD_PREFIX + rootOrgId;
        SysPropertyCacheBean notifyUrlMethodKeyProperty = CacheHelper.getSysProperty(notifyUrlMethodKey);

        if (notifyUrlKeyProperty != null && notifyUrlKeyProperty.getHasValue()) {
            notifyUrlInfo.setNotifyUrl(notifyUrlKeyProperty.getValue().toString());
        } else {
            log.warn("成绩推送信息：{} 未配置！", notifyUrlKey);
        }

        if (null != notifyUrlMethodKeyProperty && null != notifyUrlMethodKeyProperty.getValue()) {
            try {
                notifyUrlInfo.setHttpMethod(HttpMethod.valueOf(notifyUrlMethodKeyProperty.getValue().toString()));
            } catch (Exception e) {
                log.warn("成绩推送信息：{} 配置有误！", notifyUrlMethodKey);
            }
        } else {
            notifyUrlInfo.setHttpMethod(HttpMethod.POST);
        }

        return notifyUrlInfo;
    }

}
