package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import cn.com.qmth.examcloud.core.oe.admin.base.Constants;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamScorePushQueueRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.OrgScoreHandleRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamScorePushQueue;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.OrgScoreHandleEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.ExamScoreQueueStatus;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamScorePushQueueService;
import cn.com.qmth.examcloud.core.oe.admin.service.scorepushrule.ExamScorePushRuleService;
import cn.com.qmth.examcloud.support.enums.ExamProperties;
import cn.com.qmth.examcloud.support.helper.ExamCacheTransferHelper;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.support.SpringContextHolder;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * @author chenken
 * @date 2018年9月10日 上午9:26:24
 * @company QMTH
 * @description 分数推送队列接口实现
 * ("石油大学""西南财大","中国地质大学(武汉 )-入学考试")
 * 目前以上三所学校实行推分，新标准走<code>ExamScoreObtainQueueService</code>获取成绩
 */
@Service("examScorePushQueueService")
public class ExamScorePushQueueServiceImpl implements ExamScorePushQueueService {

    @Autowired
    private ExamScorePushQueueRepo examScorePushQueueRepo;

    @Autowired
    private OrgScoreHandleRepo orgScoreHandleRepo;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Override
    public void saveScoreDataInfoToQueue(Long examRecordDataId, Long examScoreId) {
        ExamRecordDataEntity examRecordData =
                GlobalHelper.getEntity(examRecordDataRepo, examRecordDataId, ExamRecordDataEntity.class);

        //得到是否推送分数属性值
        String isPushScore = ExamCacheTransferHelper.getCachedExamProperty(examRecordData.getExamId(),
                examRecordData.getStudentId(), ExamProperties.PUSH_SCORE.name()).getValue();
        if (Constants.isTrue.equals(isPushScore)) {
            Long rootOrgId = examRecordData.getRootOrgId();
            OrgScoreHandleEntity orgScoreHandle = orgScoreHandleRepo.findByRootOrgId(rootOrgId);
            if (orgScoreHandle != null) {
                String ruleServiceName = orgScoreHandle.getPushScoreRuleServiceName();
                if (StringUtils.isBlank(ruleServiceName)) {
                    return;
                }
                ExamScorePushRuleService examScorePushRuleService = (ExamScorePushRuleService) SpringContextHolder.getBean(ruleServiceName);
                //满足条件才可以进入队列,进入队列的就是需要推送的记录
                if (examScorePushRuleService.pushScoreRule(examRecordData)) {
                    ExamScorePushQueue examScorePushQueue = examScorePushQueueRepo.findByExamRecordDataId(examRecordData.getId());
                    if (examScorePushQueue == null) {
                        examScorePushQueue = new ExamScorePushQueue();
                        examScorePushQueue.setCreateTime(new Date());
                    }
                    examScorePushQueue.setExamId(examRecordData.getExamId());
                    examScorePushQueue.setExamRecordDataId(examRecordData.getId());
                    examScorePushQueue.setRootOrgId(examRecordData.getRootOrgId());
                    examScorePushQueue.setServiceName(orgScoreHandle.getPushScoreServiceName());
                    examScorePushQueue.setStatus(ExamScoreQueueStatus.PENDING);
                    examScorePushQueue.setFailedTimes(0);
                    examScorePushQueue.setUpdateTime(new Date());
                    examScorePushQueue.setExamScoreId(examScoreId);
                    examScorePushQueueRepo.save(examScorePushQueue);
                }
            }
        }
    }

}
