package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.MathUtils;
import cn.com.qmth.examcloud.core.basic.api.CourseCloudService;
import cn.com.qmth.examcloud.core.basic.api.OrgCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.OrgBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetCoursesByIdListReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetOrgMapsReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetCoursesByIdListResp;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamStatisticRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamStatisticEntity;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamStatisticService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.statistic.ExamStatisticInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.statistic.ExamStudentScoreInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.statistic.OverviewInfo;
import cn.com.qmth.examcloud.examwork.api.ExamCloudService;
import cn.com.qmth.examcloud.examwork.api.request.GetExamCourseIdsReq;
import cn.com.qmth.examcloud.examwork.api.request.GetExamCourseReq;
import cn.com.qmth.examcloud.examwork.api.response.GetExamCourseIdsResp;
import cn.com.qmth.examcloud.examwork.api.response.GetExamCourseResp;
import cn.com.qmth.examcloud.support.CacheConstants;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import com.google.common.collect.Lists;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class ExamStatisticServiceImpl implements ExamStatisticService {

    private static final Logger log = LoggerFactory.getLogger(ExamStatisticServiceImpl.class);

    @Autowired
    private ExamStatisticRepo examStatisticRepo;

    @Autowired
    private CourseCloudService courseCloudService;

    @Autowired
    private OrgCloudService orgCloudService;

    @Autowired
    private ExamCloudService examCloudService;

    @Autowired
    private RedisClient redisClient;

    @Autowired
    private JdbcTemplate jdbcTemplate;

    private final static int PASS_SCORE_LINE = 60;// 及格率默认值

    private final static int GOOD_SCORE_LINE = 90;// 优秀率默认值

    @Override
    public void refresh(Long examId) {
        final String cacheKey = CacheConstants.CACHE_OE_SCORE_STATISTIC_STATUS + examId;
        Boolean running = redisClient.get(cacheKey, Boolean.class);
        if (running != null) {
            throw new StatusException("500", "统计计算中，请稍后查询结果！");
        }

        try {
            long start = System.currentTimeMillis();
            redisClient.set(cacheKey, true, 600);// 10分钟后自动过期

            // 获取考试相关的课程ID列表
            GetExamCourseIdsReq idsReq = new GetExamCourseIdsReq();
            idsReq.setExamId(examId);
            GetExamCourseIdsResp idsResp = examCloudService.getExamCourseIds(idsReq);
            List<Long> courseIds = idsResp.getCourseIds();
            if (CollectionUtils.isEmpty(courseIds)) {
                return;
            }

            for (Long courseId : courseIds) {
                long courseStart = System.currentTimeMillis();

                GetExamCourseReq settingReq = new GetExamCourseReq();
                settingReq.setExamId(examId);
                settingReq.setCourseId(courseId);
                GetExamCourseResp settingResp = examCloudService.getExamCourseSetting(settingReq);

                int passScoreLine = PASS_SCORE_LINE, goodScoreLine = GOOD_SCORE_LINE;
                if (settingResp != null && settingResp.getBean() != null) {
                    passScoreLine = settingResp.getBean().getPassScoreLine();
                    goodScoreLine = settingResp.getBean().getGoodScoreLine();
                }

                StringBuffer sql = new StringBuffer();
                sql.append(" select es.exam_id,es.course_id,es.org_id,es.exam_student_id,");
                sql.append(" es.finished,sc.total_score,d.paper_score");
                sql.append(" from ec_oe_exam_student es");
                sql.append(" left join ec_oe_exam_student_final_score sc on sc.exam_student_id = es.exam_student_id");
                sql.append(" left join ec_oe_exam_record_data d on d.id = sc.exam_record_data_id");
                sql.append(" where es.exam_id = ").append(examId);
                sql.append(" and es.course_id = ").append(courseId);

                List<ExamStudentScoreInfo> list = jdbcTemplate.query(sql.toString(), new BeanPropertyRowMapper(ExamStudentScoreInfo.class));
                if (CollectionUtils.isEmpty(list)) {
                    log.warn("ExamStudent is empty! examId:{} courseId:{}", examId, courseId);
                    continue;
                }

                Map<Long, List<ExamStudentScoreInfo>> orgStatisticMaps = list.stream().collect(Collectors.groupingBy(ExamStudentScoreInfo::getOrgId));
                for (Map.Entry<Long, List<ExamStudentScoreInfo>> e : orgStatisticMaps.entrySet()) {
                    int allCount = 0, finishCount = 0, passScoreCount = 0, goodScoreCount = 0;
                    for (ExamStudentScoreInfo v : e.getValue()) {
                        allCount++;

                        if (v.getFinished() == null || !v.getFinished()) {
                            // 未完成考试
                            continue;
                        }

                        finishCount++;

                        // 违纪时，可能没有成绩分值
                        if (v.getTotalScore() == null) {
                            v.setTotalScore(0d);
                        }

                        if (v.getPaperScore() != null) {
                            if (v.getTotalScore() >= (v.getPaperScore() * passScoreLine) / 100d) {
                                passScoreCount++;
                            }

                            if (v.getTotalScore() >= (v.getPaperScore() * goodScoreLine) / 100d) {
                                goodScoreCount++;
                            }
                        }
                    }

                    ExamStatisticEntity result = examStatisticRepo.findByExamIdAndCourseIdAndOrgId(examId, courseId, e.getKey());
                    if (result == null) {
                        result = new ExamStatisticEntity();
                        result.setExamId(examId);
                        result.setCourseId(courseId);
                        result.setOrgId(e.getKey());
                    }
                    result.setAllCount(allCount);
                    result.setFinishCount(finishCount);
                    result.setPassScoreCount(passScoreCount);
                    result.setGoodScoreCount(goodScoreCount);
                    result.setUpdateTime(new Date());
                    examStatisticRepo.save(result);
                }

                log.warn("refresh ok! examId:{} courseId:{} cost:{}ms", examId, courseId, System.currentTimeMillis() - courseStart);
            }

            log.warn("refresh ok! examId:{} cost:{}ms", examId, System.currentTimeMillis() - start);
        } finally {
            redisClient.delete(cacheKey);
        }
    }

    @Override
    public OverviewInfo overview(Long examId, Long courseId) {
        OverviewInfo result = new OverviewInfo();

        ExamStatisticInfo examResult = new ExamStatisticInfo();
        ExamStatisticInfo courseResult = new ExamStatisticInfo();
        result.setExamResult(examResult);
        result.setCourseResult(courseResult);

        List<ExamStatisticEntity> list = examStatisticRepo.findByExamId(examId);
        if (CollectionUtils.isEmpty(list)) {
            return result;
        }

        GetCoursesByIdListReq req = new GetCoursesByIdListReq();
        req.setCourseIdList(Lists.newArrayList(courseId));
        GetCoursesByIdListResp resp = courseCloudService.getCoursesByIdList(req);
        courseResult.setCourseName(resp.getCourseList().get(0).getName());

        for (ExamStatisticEntity v : list) {
            examResult.setAllCount(examResult.getAllCount() + v.getAllCount());
            examResult.setFinishCount(examResult.getFinishCount() + v.getFinishCount());
            examResult.setPassScoreCount(examResult.getPassScoreCount() + v.getPassScoreCount());
            examResult.setGoodScoreCount(examResult.getGoodScoreCount() + v.getGoodScoreCount());
            if (v.getCourseId().equals(courseId)) {
                courseResult.setAllCount(courseResult.getAllCount() + v.getAllCount());
                courseResult.setFinishCount(courseResult.getFinishCount() + v.getFinishCount());
                courseResult.setPassScoreCount(courseResult.getPassScoreCount() + v.getPassScoreCount());
                courseResult.setGoodScoreCount(courseResult.getGoodScoreCount() + v.getGoodScoreCount());
            }
        }

        examResult.setUnFinishCount(examResult.getAllCount() - examResult.getFinishCount());
        if (examResult.getAllCount() > 0) {
            examResult.setUnFinishRate(MathUtils.percentage(examResult.getUnFinishCount(), examResult.getAllCount()));
            examResult.setPassScoreRate(MathUtils.percentage(examResult.getPassScoreCount(), examResult.getAllCount()));
            examResult.setGoodScoreRate(MathUtils.percentage(examResult.getGoodScoreCount(), examResult.getAllCount()));
        }

        courseResult.setUnFinishCount(courseResult.getAllCount() - courseResult.getFinishCount());
        if (courseResult.getAllCount() > 0) {
            courseResult.setUnFinishRate(MathUtils.percentage(courseResult.getUnFinishCount(), courseResult.getAllCount()));
            courseResult.setPassScoreRate(MathUtils.percentage(courseResult.getPassScoreCount(), courseResult.getAllCount()));
            courseResult.setGoodScoreRate(MathUtils.percentage(courseResult.getGoodScoreCount(), courseResult.getAllCount()));
        }

        return result;
    }

    @Override
    public List<ExamStatisticInfo> overviewForOrg(Long examId, Long courseId) {
        List<ExamStatisticEntity> list = examStatisticRepo.findByExamIdAndCourseId(examId, courseId);
        if (CollectionUtils.isEmpty(list)) {
            return new ArrayList<>();
        }

        List<ExamStatisticInfo> result = new ArrayList<>();

        Map<Long, List<ExamStatisticEntity>> orgStatisticMaps = list.stream().collect(Collectors.groupingBy(ExamStatisticEntity::getOrgId));

        // 获取机构信息列表
        GetOrgMapsReq orgMapsReq = new GetOrgMapsReq();
        orgMapsReq.setOrgIds(orgStatisticMaps.keySet());
        Map<Long, OrgBean> orgMaps = orgCloudService.getOrgMaps(orgMapsReq).getOrgMaps();

        for (Map.Entry<Long, List<ExamStatisticEntity>> e : orgStatisticMaps.entrySet()) {
            ExamStatisticInfo info = new ExamStatisticInfo();
            info.setOrgId(e.getKey());

            OrgBean org = orgMaps.get(e.getKey());
            if (org != null) {
                info.setOrgName(org.getName());
            }

            for (ExamStatisticEntity v : e.getValue()) {
                info.setAllCount(info.getAllCount() + v.getAllCount());
                info.setFinishCount(info.getFinishCount() + v.getFinishCount());
                info.setPassScoreCount(info.getPassScoreCount() + v.getPassScoreCount());
                info.setGoodScoreCount(info.getGoodScoreCount() + v.getGoodScoreCount());
            }

            info.setUnFinishCount(info.getAllCount() - info.getFinishCount());
            if (info.getAllCount() > 0) {
                info.setUnFinishRate(MathUtils.percentage(info.getUnFinishCount(), info.getAllCount()));
                info.setPassScoreRate(MathUtils.percentage(info.getPassScoreCount(), info.getAllCount()));
                info.setGoodScoreRate(MathUtils.percentage(info.getGoodScoreCount(), info.getAllCount()));
            }

            result.add(info);
        }

        return result;
    }

}
