package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.api.CourseCloudService;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.SystemPropertyCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.basic.api.bean.StudentBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetCourseReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetStudentReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetSystemPropertyReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetCourseResp;
import cn.com.qmth.examcloud.core.basic.api.response.GetStudentResp;
import cn.com.qmth.examcloud.core.basic.api.response.GetSystemPropertyResp;
import cn.com.qmth.examcloud.core.oe.admin.service.GainBaseDataService;
import cn.com.qmth.examcloud.examwork.api.ExamStudentCloudService;
import cn.com.qmth.examcloud.examwork.api.bean.ExamStudentBean;
import cn.com.qmth.examcloud.examwork.api.request.GetExamStudentReq;
import cn.com.qmth.examcloud.examwork.api.response.GetExamStudentResp;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.OrgCacheBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @author chenken
 * @date 2018年10月17日 下午2:28:53
 * @company QMTH
 * @description 获取基础数据接口实现
 */
@Service("gainBaseDataService")
public class GainBaseDataServiceImpl implements GainBaseDataService {

    @Autowired
    private CourseCloudService courseCloudService;

    @Autowired
    private StudentCloudService studentCloudService;

    @Autowired
    private ExamStudentCloudService examStudentCloudService;

    @Autowired
    private SystemPropertyCloudService systemPropertyCloudService;

    @Override
    public CourseBean getCourseBean(Long courseId, Long rootOrgId) {
        GetCourseReq getCourseReq = new GetCourseReq();
        getCourseReq.setId(courseId);
        getCourseReq.setRootOrgId(rootOrgId);
        GetCourseResp getCourseResp = courseCloudService.getCourse(getCourseReq);
        CourseBean courseBean = getCourseResp.getCourseBean();
        return courseBean;
    }

    @Override
    public CourseBean getCourseBean(String courseCode, Long rootOrgId) {
        GetCourseReq getCourseReq = new GetCourseReq();
        getCourseReq.setRootOrgId(rootOrgId);
        getCourseReq.setCode(courseCode);
        GetCourseResp getCourseResp = courseCloudService.getCourse(getCourseReq);
        CourseBean courseBean = getCourseResp.getCourseBean();
        return courseBean;
    }

    @Override
    public OrgCacheBean getOrgBean(Long orgId) {
        OrgCacheBean org = CacheHelper.getOrg(orgId);
        if (org == null) {
            throw new StatusException("100001", "找不到id为" + orgId + "的组织机构信息");
        }
        return org;
    }

    @Override
    public StudentBean getStudentBean(Long studentId) {
        GetStudentReq getStudentReq = new GetStudentReq();
        getStudentReq.setStudentId(studentId);
        GetStudentResp getStudentResp = studentCloudService.getStudent(getStudentReq);
        return getStudentResp.getStudentInfo();
    }

    /**
     * 根据学号获取学生信息
     *
     * @param studentCode
     * @return
     */
    @Override
    public StudentBean getStudentBean(String studentCode, Long rootOrgId) {
        GetStudentReq getStudentReq = new GetStudentReq();
        getStudentReq.setStudentCode(studentCode);
        getStudentReq.setRootOrgId(rootOrgId);
        GetStudentResp getStudentResp = studentCloudService.getStudent(getStudentReq);
        return getStudentResp.getStudentInfo();
    }

    @Override
    public ExamStudentBean getExamStudentBean(Long examStudentId) {
        GetExamStudentReq req = new GetExamStudentReq();
        req.setExamStudentId(examStudentId);
        GetExamStudentResp getExamStudentResp = examStudentCloudService.getExamStudent(req);
        return getExamStudentResp.getExamStudentBean();
    }

    @Override
    public String getSysConfig(String key) {
        GetSystemPropertyReq getSysConfigReq = new GetSystemPropertyReq();
        getSysConfigReq.setKey(key);
        GetSystemPropertyResp getSysConfigResp = systemPropertyCloudService.getSystemProperty(getSysConfigReq);
        return getSysConfigResp.getValue();
    }

}
