package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import cn.com.qmth.examcloud.api.commons.enums.AdminOperateType;
import cn.com.qmth.examcloud.api.commons.enums.CallType;
import cn.com.qmth.examcloud.api.commons.enums.ExamSpecialSettingsType;
import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.JsonMapper;
import cn.com.qmth.examcloud.core.oe.admin.base.Constants;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordFileAnswerRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamStudentRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordFileAnswerEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamStudentEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.core.oe.admin.service.*;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.OfflineExamCourseInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamRecordDataBean;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamRecordFileAnswerInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.ExamStudentBean;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.uploadfile.FileInfo;
import cn.com.qmth.examcloud.core.questions.api.ExtractConfigCloudService;
import cn.com.qmth.examcloud.core.questions.api.request.GetPaperReq;
import cn.com.qmth.examcloud.core.questions.api.response.GetPaperResp;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultPaper;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionGroup;
import cn.com.qmth.examcloud.reports.commons.bean.AdminOperateReport;
import cn.com.qmth.examcloud.reports.commons.util.ReportsUtil;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.*;
import cn.com.qmth.examcloud.support.enums.ExamProperties;
import cn.com.qmth.examcloud.support.filestorage.FileStorageUtil;
import cn.com.qmth.examcloud.support.helper.ExamCacheTransferHelper;
import cn.com.qmth.examcloud.web.filestorage.FileStoragePathEnvInfo;
import cn.com.qmth.examcloud.web.filestorage.YunPathInfo;
import com.google.common.collect.Lists;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;

/**
 * @author chenken
 * @date 2018年9月5日 下午3:24:44
 * @company QMTH
 * @description 离线考试服务实现
 */
@Service("offlineExamService")
public class OfflineExamServiceImpl implements OfflineExamService {

    private static final Logger log = LoggerFactory.getLogger(OfflineExamServiceImpl.class);

    @Autowired
    private ExamStudentRepo examStudentRepo;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private GainBaseDataService gainBaseDataService;

    @Autowired
    private ExtractConfigCloudService extractConfigCloudService;

    @Autowired
    private ExamRecordDataService examRecordDataService;

    @Autowired
    private ExamScoreService examScoreService;

    @Autowired
    private ExamRecordForMarkingService examRecordForMarkingService;

    @Autowired
    private ExamRecordFileAnswerRepo examRecordFileAnswerRepo;

    @Autowired
    private ExamStudentService examStudentService;

    @Value("${app.upyun.uploadUrl}")
    private String upyunUploadUrl;

    @Override
    public List<OfflineExamCourseInfo> getOfflineCourse(Long studentId) {
        List<ExamStudentEntity> examStudents = examStudentRepo.findByStudentId(studentId);

        List<OfflineExamCourseInfo> offlineExamCourseInfoList = new ArrayList<>();
        for (ExamStudentEntity examStudent : examStudents) {
            if (!examStudent.getEnable()) {
                log.warn("ExamStudent is disable, id is " + examStudent.getExamStudentId());
                continue;
            }

            ExamSettingsCacheBean examBean =
                    ExamCacheTransferHelper.getCachedExam(examStudent.getExamId(), studentId, examStudent.getExamStageId());
            if (!ExamType.OFFLINE.name().equals(examBean.getExamType())) {
                continue;
            }
            if (!examBean.getEnable() || (examBean.getExamLimit() != null && examBean.getExamLimit())) {
                continue;
            }
            if (new Date().before(examBean.getBeginTime()) || examBean.getEndTime().before(new Date())) {
                continue;
            }
            offlineExamCourseInfoList.add(toOfflineExamCourse(examStudent, examBean));
        }
        return offlineExamCourseInfoList;
    }

    private OfflineExamCourseInfo toOfflineExamCourse(ExamStudentEntity examStudent, ExamSettingsCacheBean examBean) {
        OfflineExamCourseInfo offlineExamCourseInfo = new OfflineExamCourseInfo();

        CourseCacheBean courseBean = ExamCacheTransferHelper.getCachedCourse(examStudent.getCourseId());
        offlineExamCourseInfo.setCourseCode(courseBean.getCode());
        offlineExamCourseInfo.setCourseLevel(courseBean.getLevel());
        offlineExamCourseInfo.setCourseName(courseBean.getName());
        offlineExamCourseInfo.setExamId(examBean.getId());
        offlineExamCourseInfo.setExamName(examBean.getName());
        offlineExamCourseInfo.setSpecialtyName(examStudent.getSpecialtyName());
        offlineExamCourseInfo.setExamStudentId(examStudent.getExamStudentId());
        offlineExamCourseInfo.setStudentCode(examStudent.getStudentCode());
        offlineExamCourseInfo.setStudentName(examStudent.getStudentName());
        offlineExamCourseInfo.setStartTime(examBean.getBeginTime());
        offlineExamCourseInfo.setEndTime(examBean.getEndTime());
        Date nowDate = new Date();
        if (nowDate.getTime() > offlineExamCourseInfo.getStartTime().getTime()
                && nowDate.getTime() < offlineExamCourseInfo.getEndTime().getTime()) {
            offlineExamCourseInfo.setIsvalid(true);
        } else {
            offlineExamCourseInfo.setIsvalid(false);
        }

        OrgCacheBean orgBean = gainBaseDataService.getOrgBean(examStudent.getOrgId());
        offlineExamCourseInfo.setOrgName(orgBean.getName());

        List<ExamRecordDataEntity> examRecords = examRecordDataRepo.findByExamStudentId(examStudent.getExamStudentId());
        if (examRecords.size() > 0) {
            ExamRecordDataEntity examRecordDataEntity = examRecords.get(0);
            offlineExamCourseInfo.setExamRecordDataId(examRecordDataEntity.getId());
            offlineExamCourseInfo.setStatus(examRecordDataEntity.getExamRecordStatus());
            offlineExamCourseInfo.setPaperId(examRecordDataEntity.getBasePaperId());
            if (examRecordDataEntity.getExamRecordStatus() == ExamRecordStatus.EXAM_END) {
                List<ExamRecordFileAnswerEntity> fileAnswerList = examRecordFileAnswerRepo.findByExamRecordDataId(examRecordDataEntity.getId());
                offlineExamCourseInfo.setOfflineFiles(getOfflineFilesFrom(fileAnswerList));
            }
        }
        return offlineExamCourseInfo;
    }

    private List<ExamRecordFileAnswerInfo> getOfflineFilesFrom(List<ExamRecordFileAnswerEntity> fileAnswerList) {
        List<ExamRecordFileAnswerInfo> resultList = new ArrayList<>();
        for (ExamRecordFileAnswerEntity entity : fileAnswerList) {
            ExamRecordFileAnswerInfo info = new ExamRecordFileAnswerInfo();
            info.setId(entity.getId());
            info.setExamRecordDataId(entity.getExamRecordDataId());
            info.setOfflineFileUrl(FileStorageUtil.realPath(entity.getFileUrl()));
            info.setOfflineFileName(entity.getFileName());
            info.setOriginalFileName(entity.getOriginalFileName());
            info.setFileType(entity.getFileType());
            info.setSuffix(entity.getSuffix());
            info.setProperties(entity.getProperties());
            resultList.add(info);
        }

        return resultList;
    }

    private double parsePaperScore(DefaultPaper defaultPaper) {
        double paperScore = 0d;
        if (org.apache.commons.collections.CollectionUtils.isNotEmpty(defaultPaper.getQuestionGroupList())) {
            for (DefaultQuestionGroup group : defaultPaper.getQuestionGroupList()) {
                paperScore += group.getGroupScore();
            }
        }
        return paperScore;
    }

    @Override
    public void startOfflineExam(Long examStudentId) {
        Check.isNull(examStudentId, "examStudentId不能为空");

        boolean isEnable = examStudentService.isEnableExamStudent(examStudentId);
        if (!isEnable) {
            throw new StatusException("000500", "当前考生已禁用！");
        }

        SysPropertyCacheBean stuClientLoginLimit = CacheHelper.getSysProperty("STU_CLIENT_LOGIN_LIMIT");
        boolean stuClientLoginLimitBoolean = false;
        if (stuClientLoginLimit.getHasValue()) {
            stuClientLoginLimitBoolean = Boolean.parseBoolean(stuClientLoginLimit.getValue().toString());
        }

        if (stuClientLoginLimitBoolean) {
            throw new StatusException("1001", "系统维护中... ...");
        }

        List<ExamRecordDataEntity> examRecordList = examRecordDataRepo.findByExamStudentId(examStudentId);
        if (examRecordList != null && examRecordList.size() > 0) {
            throw new StatusException("1002", "已经存在examStudentId=" + examStudentId + "的离线考试记录");
        }

        //获取考生信息
        ExamStudentEntity examStudentEntity = examStudentRepo.findByExamStudentId(examStudentId);
        ExamStudentBean bean = of(examStudentEntity);

        //检查并获取课程信息
        CourseCacheBean courseBean = checkCourse(bean);

        //检查并获取考试信息
        ExamSettingsCacheBean examBean = checkExam(bean);

        //获取题库试卷结构
        GetPaperReq getPaperReq = new GetPaperReq();
        getPaperReq.setExamId(examStudentEntity.getExamId());
        getPaperReq.setCourseCode(courseBean.getCode());
        getPaperReq.setGroupCode(examStudentEntity.getPaperType());
        GetPaperResp getPaperResp = extractConfigCloudService.getPaper(getPaperReq);

        // 是否采用千人千卷规则 （目前离线考试不支持千卷模式，默认false）
        boolean randomPaper = CallType.RANDOM_PAPER.name().equals(examBean.getCallType());

        DefaultPaper defaultPaper = getPaperResp.getDefaultPaper();
        double paperScore = this.parsePaperScore(defaultPaper);

        //生成考试记录
        ExamRecordDataEntity examRecordData = examRecordDataService.createOfflineExamRecordData(
                bean, examBean, courseBean, getPaperResp.getPaperId(), defaultPaper.getFullyObjective(),
                randomPaper, paperScore);

        //生成分数
        examScoreService.createExamScoreWithOffline(examRecordData.getId());

        //更新考生
        examStudentRepo.updateExamStudentFinished(examStudentId);
    }

    private CourseCacheBean checkCourse(ExamStudentBean bean) {
        CourseCacheBean courseBean = ExamCacheTransferHelper.getCachedCourse(bean.getCourseId());
        if (!courseBean.getEnable()) {
            throw new StatusException("2001", "该课程已被禁用");
        }
        return courseBean;
    }

    private ExamSettingsCacheBean checkExam(ExamStudentBean bean) {
        Long examId = bean.getExamId();
        Long studentId = bean.getStudentId();
        ExamSettingsCacheBean examBean = ExamCacheTransferHelper.getCachedExam(examId, studentId, bean.getExamStageId());

        //如果启用了了特殊设置，并且无特殊设置时结束考试 配置 设置为true..且实际未设置特殊设置则不允许考试
        ExamPropertyCacheBean limitedIfNoSpecialSettings = ExamCacheTransferHelper.getDefaultCachedExamProperty(examId,
                ExamProperties.LIMITED_IF_NO_SPECIAL_SETTINGS.toString());
        if (examBean.getSpecialSettingsEnabled() &&
                (limitedIfNoSpecialSettings.getHasValue() && Boolean.valueOf(limitedIfNoSpecialSettings.getValue()))) {

            //学生特殊设置开启未配置，不允许考试
            if (examBean.getSpecialSettingsType() == ExamSpecialSettingsType.STUDENT_BASED) {
                ExamStudentSettingsCacheBean specialSettings = CacheHelper.getExamStudentSettings(examId, studentId);
                if (!specialSettings.getHasValue()) {
                    throw new StatusException("3001", "考试配置未完成，不允许考试");
                }
            }

            //机构特殊设置开启未配置，不允许考试
            if (examBean.getSpecialSettingsType() == ExamSpecialSettingsType.ORG_BASED) {
                //需求调整，所有的组织机构取学生表所关联的orgId
                Long orgId = CacheHelper.getStudent(studentId).getOrgId();
                ExamOrgSettingsCacheBean specialSettings = CacheHelper.getExamOrgSettings(examId, orgId);
                if (!specialSettings.getHasValue()) {
                    throw new StatusException("3002", "考试配置未完成，不允许考试");
                }
            }
        }

        if (!examBean.getEnable() || (examBean.getExamLimit() != null && examBean.getExamLimit())) {
            throw new StatusException("3003", "暂无考试资格,请与学校老师联系");
        }
        if (new Date().before(examBean.getBeginTime())) {
            throw new StatusException("3004", "考试未开始");
        }
        if (examBean.getEndTime().before(new Date())) {
            throw new StatusException("3005", "本次考试已结束");
        }
        return examBean;
    }

    @Override
    public void submitPaper(Long examRecordDataId, FileInfo fileInfo, Long userId) {
        if (fileInfo.getFileSize() > Constants.ANSWER_FILE_MAX_SIZE * Constants.M_SIZE) {
            throw new StatusException("400402", "文件大小不能超过" + Constants.ANSWER_FILE_MAX_SIZE + "M");
        }

        String fileType;
        if (".zip".equals(fileInfo.getFileSuffix())) {
            fileType = "ZIP";
        } else if (".pdf".equals(fileInfo.getFileSuffix())) {
            fileType = "PDF";
        } else if (".jpg.jpeg.png".contains(fileInfo.getFileSuffix())) {
            fileType = "IMAGE";
        } else {
            fileType = "UNKNOWN";
        }

        this.batchSubmitPaper(examRecordDataId, Lists.newArrayList(fileInfo), fileType, userId);
    }

    @Override
    @Transactional
    public void batchSubmitPaper(Long examRecordDataId, List<FileInfo> fileInfoList, String fileType, Long userId) {
        Check.isNull(examRecordDataId, "examRecordDataId不能为空");

        SysPropertyCacheBean stuClientLoginLimit = CacheHelper.getSysProperty("STU_CLIENT_LOGIN_LIMIT");
        Boolean stuClientLoginLimitBoolean = false;
        if (stuClientLoginLimit.getHasValue()) {
            stuClientLoginLimitBoolean = Boolean.valueOf(stuClientLoginLimit.getValue().toString());
        }
        if (stuClientLoginLimitBoolean) {
            throw new StatusException("4001", "系统维护中... ...");
        }

        Optional<ExamRecordDataEntity> optional = examRecordDataRepo.findById(examRecordDataId);
        if (!optional.isPresent()) {
            throw new StatusException("400403", "考试记录不存在");
        }

        ExamRecordDataEntity examRecordData = optional.get();
        if (ExamType.OFFLINE != examRecordData.getExamType()) {
            throw new StatusException("400403", "非离线考试");
        }

        if (!examStudentService.isEnableExamStudent(examRecordData.getExamStudentId())) {
            throw new StatusException("400403", "当前考生已禁用");
        }

        // 当前考试允许的附件类型，示例：["ZIP","PDF","IMAGE"]
        ExamPropertyCacheBean examProperty = ExamCacheTransferHelper.getCachedExamProperty(examRecordData.getExamId(),
                examRecordData.getStudentId(), ExamProperties.OFFLINE_UPLOAD_FILE_TYPE.name());
        String[] settingFileTypes = new JsonMapper().toArray(examProperty.getValue(), String.class);
        if (ArrayUtils.isEmpty(settingFileTypes)) {
            throw new StatusException("400403", "当前考试设置不允许上传附件");
        }

        String uploadFileType = fileType.toUpperCase();
        if (!ArrayUtils.contains(settingFileTypes, uploadFileType)) {
            throw new StatusException("400403", "当前考试允许上传文件格式为:" + examProperty.getValue());
        }

        if (("ZIP".equals(uploadFileType) || "PDF".equals(uploadFileType)) && fileInfoList.size() > 1) {
            throw new StatusException("400403", "上传" + uploadFileType + "类型文件时只能传一个文件");
        }

        ExamRecordDataBean bean = of(examRecordData);

        // 上传文件
        for (FileInfo fileInfo : fileInfoList) {
            if (!matchFileTypes(new String[]{uploadFileType}, fileInfo.getFileSuffix())) {
                throw new StatusException("400401", "请上传【" + uploadFileType + "】类型的文件");
            }

            String fileNewName = createOfflineFileName(bean) + fileInfo.getFileSuffix();
            String relativePath = upyunUploadUrl + examRecordData.getExamId() + "/" + fileNewName;

            FileStoragePathEnvInfo env = new FileStoragePathEnvInfo();
            env.setRootOrgId(String.valueOf(examRecordData.getRootOrgId()));
            env.setRelativePath(relativePath);
            YunPathInfo uploadResult = FileStorageUtil.saveFile("offlineFile", env, fileInfo.getFileBytes(), false);

            fileInfo.setFilePath(uploadResult.getRelativePath());
            fileInfo.setFileName(fileNewName);
            fileInfo.setFileType(fileType);
        }

        // 删除原作答文件记录
        List<ExamRecordFileAnswerEntity> fileAnswerList = examRecordFileAnswerRepo.findByExamRecordDataId(examRecordDataId);
        if (CollectionUtils.isNotEmpty(fileAnswerList)) {
            for (ExamRecordFileAnswerEntity fileAnswer : fileAnswerList) {
                examRecordFileAnswerRepo.delete(fileAnswer);
                FileStorageUtil.deleteFile(fileAnswer.getFileUrl());
            }
        }

        // 保存新作答文件记录
        for (FileInfo fileInfo : fileInfoList) {
            saveExamRecordFileAnswer(examRecordDataId, fileInfo.getOriginalFileName(), fileInfo.getFileName(), fileInfo.getFilePath(),
                    fileInfo.getFileType().toLowerCase(), fileInfo.getFileSuffix().replaceFirst(".", ""));
        }

        fileInfoList.clear();

        examRecordForMarkingService.saveOffLineExamRecordForMarking(bean);

        //更新考试记录状态，交卷(上传)时间
        examRecordDataRepo.updateExamRecordStatusById(examRecordDataId, ExamRecordStatus.EXAM_END, new Date());

        ReportsUtil.report(new AdminOperateReport(examRecordData.getRootOrgId(), userId, AdminOperateType.TYPE21.getDesc(),
                "考试记录ID:" + examRecordDataId + " 考生ID:" + examRecordData.getExamStudentId()));
    }

    private boolean matchFileTypes(String[] settingFileTypes, String fileSuffix) {
        if (ArrayUtils.isEmpty(settingFileTypes) || StringUtils.isEmpty(fileSuffix)) {
            return false;
        }

        for (String fileType : settingFileTypes) {
            if ("IMAGE".equals(fileType)) {
                if (".jpg.jpeg.png".contains(fileSuffix)) {
                    return true;
                }
            } else {
                // ZIP、PDF
                if (("." + fileType).equalsIgnoreCase(fileSuffix)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 保存考试记录文件作答
     *
     * @param examRecordDataId
     * @param originalFileName 原始文件名称
     * @param fileName         新文件名称
     * @param relativePath
     * @param fileType
     * @param fileSuffix
     */
    private void saveExamRecordFileAnswer(Long examRecordDataId, String originalFileName, String fileName,
                                          String relativePath, String fileType, String fileSuffix) {
        ExamRecordFileAnswerEntity entity = new ExamRecordFileAnswerEntity();
        entity.setExamRecordDataId(examRecordDataId);
        entity.setOriginalFileName(originalFileName);
        entity.setFileName(fileName);
        entity.setFileUrl(relativePath);
        entity.setFileType(fileType);
        entity.setSuffix(fileSuffix);
        examRecordFileAnswerRepo.save(entity);
    }

    private String createOfflineFileName(ExamRecordDataBean examRecordData) {
        long currentTime = System.currentTimeMillis();

        long orgId = examRecordData.getOrgId();
        OrgCacheBean orgBean = gainBaseDataService.getOrgBean(orgId);
        long courseId = examRecordData.getCourseId();
        CourseCacheBean courseBean = ExamCacheTransferHelper.getCachedCourse(courseId);
        return orgBean.getCode() + "_" +
                examRecordData.getStudentCode() + "_" +
                examRecordData.getStudentName() + "_" +
                courseBean.getCode() + "_" + currentTime;
    }

    private ExamRecordDataBean of(ExamRecordDataEntity record) {
        ExamRecordDataBean info = new ExamRecordDataBean();
        //封装基础数据
        info.setId(record.getId());
        info.setExamId(record.getExamId());
        info.setExamType(record.getExamType());
        info.setExamStudentId(record.getExamStudentId());
        info.setStudentId(record.getStudentId());
        info.setStudentCode(record.getStudentCode());
        info.setStudentName(record.getStudentName());
        info.setIdentityNumber(record.getIdentityNumber());
        info.setCourseId(record.getCourseId());

        info.setCourseLevel(record.getCourseLevel());
        info.setOrgId(record.getOrgId());

        info.setRootOrgId(record.getRootOrgId());
        info.setBasePaperId(record.getBasePaperId());
        info.setPaperStructId(record.getPaperStructId());
        info.setInfoCollector(record.getInfoCollector());
        info.setExamRecordQuestionsId(record.getExamRecordQuestionsId());
        info.setExamRecordStatus(record.getExamRecordStatus());
        info.setStartTime(record.getStartTime());
        info.setEndTime(record.getEndTime());
        info.setCleanTime(record.getCleanTime());
        info.setIsWarn(record.getIsWarn());
        info.setIsAudit(record.getIsAudit());
        info.setIsIllegality(record.getIsIllegality());

        info.setUsedExamTime(record.getUsedExamTime());
        info.setExamOrder(record.getExamOrder());
        info.setIsReexamine(record.getIsReexamine());
        info.setIsContinued(record.getIsContinued());
        info.setContinuedCount(record.getContinuedCount());
        info.setIsAllObjectivePaper(record.getIsAllObjectivePaper());
        info.setIsExceed(record.getIsExceed());
        info.setFaceSuccessCount(record.getFaceSuccessCount());
        info.setFaceFailedCount(record.getFaceFailedCount());
        info.setFaceStrangerCount(record.getFaceStrangerCount());
        info.setFaceTotalCount(record.getFaceTotalCount());
        info.setFaceSuccessPercent(record.getFaceSuccessPercent());
        info.setBaiduFaceLivenessSuccessPercent(record.getBaiduFaceLivenessSuccessPercent());
        info.setFaceVerifyResult(record.getFaceVerifyResult());
        info.setFaceLandmarkVal(record.getFaceLandmarkVal());
        return info;
    }

    private ExamStudentBean of(ExamStudentEntity et) {
        ExamStudentBean b = new ExamStudentBean();
        b.setId(et.getId());
        b.setExamStudentId(et.getExamStudentId());
        b.setExamId(et.getExamId());
        b.setStudentId(et.getStudentId());
        b.setStudentCode(et.getStudentCode());
        b.setStudentName(et.getStudentName());
        b.setIdentityNumber(et.getIdentityNumber());
        b.setCourseId(et.getCourseId());
        b.setCourseCode(et.getCourseCode());
        b.setCourseLevel(et.getCourseLevel());
        b.setOrgId(et.getOrgId());
        b.setRootOrgId(et.getRootOrgId());
        b.setSpecialtyCode(et.getSpecialtyCode());
        b.setSpecialtyName(et.getSpecialtyName());
        b.setPaperType(et.getPaperType());
        b.setInfoCollector(et.getInfoCollector());
        b.setFinished(et.getFinished());
        b.setUsedNum(et.getUsedNum());
        b.setExtraNum(et.getExtraNum());
        b.setGrade(et.getGrade());
        b.setEnable(et.getEnable());
        b.setExamStageId(et.getExamStageId());
        b.setExamStageOrder(et.getExamStageOrder());

        return b;
    }

}
