package cn.com.qmth.examcloud.core.oe.admin.service.pushscore.cugr;

import cn.com.qmth.examcloud.commons.util.DateUtil;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamScorePushQueueRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamScorePushQueue;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.ExamScoreQueueStatus;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamScorePushService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamScoreQueueService;
import cn.com.qmth.examcloud.core.oe.admin.service.GainBaseDataService;
import cn.com.qmth.examcloud.support.cache.bean.CourseCacheBean;
import cn.com.qmth.examcloud.support.helper.ExamCacheTransferHelper;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

/**
 * @author chenken
 * @date 2018年10月18日 下午4:08:51
 * @company QMTH
 * @description 地质大学（武汉）入学  orgCode:cugr.ecs.qmth.com.cn
 */
@Service("cugrExamScorePushServiceImpl")
public class CugrExamScorePushServiceImpl implements ExamScorePushService {

    private static final Logger log = LoggerFactory.getLogger(CugrExamScorePushServiceImpl.class);

    @Value("${$cug_wh_push_score_key}")
    private String pushScoreKey;

    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Autowired
    private ExamScorePushQueueRepo examScorePushQueueRepo;

    @Autowired
    private ExamScoreQueueService examScoreQueueService;

    @Autowired
    private GainBaseDataService gainBaseDataService;

    private static final String CUGR_URL = "http://202.114.196.58/busInfcCenter/setexamscore.infc";

    @Override
    public void scorePush(ExamScorePushQueue examScorePushQueue) {
        CloseableHttpResponse httpResponse = null;
        CloseableHttpClient httpClient = HttpClients.createDefault();
        String result = "";
        try {
            String url = getUrl(examScorePushQueue);
            log.info("cugr_push_score_url:" + url);
            HttpPost httpPost = new HttpPost(url);
            httpResponse = httpClient.execute(httpPost);
            HttpEntity responseEntity = httpResponse.getEntity();
            BufferedReader reader = new BufferedReader(new InputStreamReader(responseEntity.getContent()));
            StringBuffer buffer = new StringBuffer();
            String str = "";
            while (StringUtils.isNoneBlank((str = reader.readLine()))) {
                buffer.append(str);
            }
            result = buffer.toString();
            JSONObject jsonObject = new JSONObject(result);
            if (jsonObject.has("describe") && "success".equals(jsonObject.getString("describe"))) {
                examScoreQueueService.saveExamScorePushQueue(result, examScorePushQueue, ExamScoreQueueStatus.PROCESS_SUCCESS);
            } else {
                examScoreQueueService.saveExamScorePushQueue(result, examScorePushQueue, ExamScoreQueueStatus.PROCESS_FAILED);
            }
        } catch (Exception e) {
            log.error(examScorePushQueue.getId() + "成绩推送失败", e);
            examScoreQueueService.saveExamScorePushQueue(e.getMessage(), examScorePushQueue, ExamScoreQueueStatus.PROCESS_FAILED);
        } finally {
            try {
                if (httpClient != null) {
                    httpClient.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            try {
                if (httpResponse != null) {
                    httpResponse.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private String getUrl(ExamScorePushQueue examScorePushQueue) {
        long examRecordDataId = examScorePushQueue.getExamRecordDataId();
        long scoreId = examScorePushQueue.getExamScoreId();

        String examRecordsql = "select identity_number,course_id from ec_oe_exam_record_data where id=" + examRecordDataId;
        String scoreSql = "select total_score from  ec_oe_exam_score where id=" + scoreId;

        List<TmpExamRecord> examRecordList = jdbcTemplate.query(examRecordsql, new RowMapper<TmpExamRecord>() {
            @Override
            public TmpExamRecord mapRow(ResultSet rs, int rowNum) throws SQLException {
                TmpExamRecord tmpExamRecord = new TmpExamRecord();
                tmpExamRecord.setIdentityNumber(rs.getString("identity_number"));
                tmpExamRecord.setCourseId(rs.getLong("course_id"));
                return tmpExamRecord;
            }
        });

        TmpExamRecord examRecord = examRecordList.get(0);

        CourseCacheBean courseBean = ExamCacheTransferHelper.getCachedCourse(examRecord.getCourseId());

        double totalScore = jdbcTemplate.queryForObject(scoreSql, Double.class);
        return buildUrl(totalScore, courseBean.getCode(), examRecord.getIdentityNumber());
    }

    private String buildUrl(Double totalScore, String courseCode, String identityNumber) {
        String now = DateUtil.now("yyyy-MM-dd%20HH:mm:ss");

        StringBuilder sb = new StringBuilder();
        sb.append(courseCode).append(",").append(identityNumber).append(",").append("'cugexma'").append(pushScoreKey);
        String sign = null;
        try {
            sign = DigestUtils.md5Hex(sb.toString().getBytes("utf-8"));
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }
        StringBuilder params = new StringBuilder();
        params.append("new_date").append("=").append(now);
        params.append("&").append("total_score").append("=").append(totalScore);
        params.append("&").append("rec_exam_course").append("=").append(courseCode);
        params.append("&").append("card_no").append("=").append(identityNumber);
        params.append("&").append("md5sign").append("=").append(sign);

        return CUGR_URL + "?" + params.toString();
    }

}

class TmpExamRecord {

    private String identityNumber;

    private Long courseId;

    public String getIdentityNumber() {
        return identityNumber;
    }

    public void setIdentityNumber(String identityNumber) {
        this.identityNumber = identityNumber;
    }

    public Long getCourseId() {
        return courseId;
    }

    public void setCourseId(Long courseId) {
        this.courseId = courseId;
    }

}

