package cn.com.qmth.examcloud.core.oe.admin.service.pushscore.cup;

import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamScorePushQueueRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamScoreRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamStudentRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamScoreEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamScorePushQueue;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamStudentEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.ExamScoreQueueStatus;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamScorePushService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamScoreQueueService;
import cn.com.qmth.examcloud.core.oe.admin.service.pushscore.bean.ExamScorePushInfo;
import cn.com.qmth.examcloud.support.cache.bean.CourseCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.ExamSettingsCacheBean;
import cn.com.qmth.examcloud.support.helper.ExamCacheTransferHelper;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URI;

/**
 * @author chenken
 * @date 2018年10月18日 下午4:09:26
 * @company QMTH
 * @description 石油大学  orgCode:cup.ecs.qmth.com.cn
 */
@Service("cupExamScorePushServiceImpl")
public class CupExamScorePushServiceImpl implements ExamScorePushService {

    private static final Logger log = LoggerFactory.getLogger(CupExamScorePushServiceImpl.class);

    @Autowired
    private ExamScoreRepo examScoreRepo;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private ExamStudentRepo examStudentRepo;

    @Autowired
    private ExamScorePushQueueRepo examScorePushQueueRepo;

    @Autowired
    private ExamScoreQueueService examScoreQueueService;

    private String pushUrlSchema = "http";

    private String pushUrlHost = "www.cupde.cn";

    private String pushUrlPort = "80";

    @Override
    public void scorePush(ExamScorePushQueue examScorePushQueue) {
        log.info("开始推送石油大学成绩:" + examScorePushQueue.getId());
        Long scoreId = examScorePushQueue.getExamScoreId();
        ExamScoreEntity examScore = GlobalHelper.getEntity(examScoreRepo, scoreId, ExamScoreEntity.class);
        ExamRecordDataEntity examRecordData = GlobalHelper.getEntity(examRecordDataRepo, examScore.getExamRecordDataId(), ExamRecordDataEntity.class);

        ExamStudentEntity examStudent = examStudentRepo.findByExamStudentId(examRecordData.getExamStudentId());
        if (examScore == null || examRecordData == null || examStudent == null) {
            return;
        }

        ExamScorePushInfo examScorePushInfo = buildexamScorePushInfo(examScore, examRecordData, examStudent);
        String courseCode = examScorePushInfo.getCourseCode();
        //将Z000001变成000001
        if (courseCode.startsWith("Z") || courseCode.startsWith("Q") || courseCode.startsWith("S")) {
            examScorePushInfo.setCourseCode(courseCode.substring(1, courseCode.length()));
        } else {
            examScorePushInfo.setCourseCode(courseCode);
        }
        CloseableHttpResponse httpResponse = null;
        CloseableHttpClient httpClient = null;
        try {
            String scoreStr = JsonUtil.toJson(examScorePushInfo);
            log.info("石油大学成绩数据：{}", scoreStr);
            RequestConfig config = RequestConfig.custom()
                    .setConnectTimeout(5000)
                    .setConnectionRequestTimeout(5000)
                    .setSocketTimeout(5000)
                    .build();
            httpClient = HttpClientBuilder
                    .create()
                    .setDefaultRequestConfig(config)
                    .build();
            URI uri = new URIBuilder()
                    .setScheme(pushUrlSchema)
                    .setHost(pushUrlHost)
                    .setPort(Integer.parseInt(pushUrlPort))
                    .setPath("/entity/learning/interface/stuHomeworkScore_syncFinalExamScore.action")
                    .setParameter("loginId", "bjsy")
                    .setParameter("password", "1111")
                    .setParameter("examRemark", examScorePushInfo.getExamName())
                    .setParameter("examStuRemark", "recruit")
                    .setParameter("scoreJsonStr", scoreStr)//通过查询参数的方式传递
                    .build();
            HttpPost httpPost = new HttpPost(uri);
            httpResponse = httpClient.execute(httpPost);
            HttpEntity responseEntity = httpResponse.getEntity();
            String result = "";
            int statusCode = httpResponse.getStatusLine().getStatusCode();
            if (statusCode == 200) {
                BufferedReader reader = new BufferedReader(new InputStreamReader(responseEntity.getContent()));
                StringBuffer buffer = new StringBuffer();
                String str = "";
                while (StringUtils.isNoneBlank((str = reader.readLine()))) {
                    buffer.append(str);
                }
                result = buffer.toString();
            }
            log.info("推送石油大学成绩:" + examScorePushQueue.getId() + "结果:" + result);
            JSONObject jsonObject = new JSONObject(result);
            //推送成功
            if (jsonObject.has("success") && "true".equals(jsonObject.getString("success"))) {
                examScoreQueueService.saveExamScorePushQueue(result, examScorePushQueue, ExamScoreQueueStatus.PROCESS_SUCCESS);
            } else {
                examScoreQueueService.saveExamScorePushQueue(result, examScorePushQueue, ExamScoreQueueStatus.PROCESS_FAILED);
            }
        } catch (Exception e) {
            log.error(examScorePushQueue.getId() + "成绩推送失败", e);
            examScoreQueueService.saveExamScorePushQueue(e.getMessage(), examScorePushQueue, ExamScoreQueueStatus.PROCESS_FAILED);
        } finally {
            if (httpResponse != null) {
                try {
                    httpResponse.close();
                } catch (IOException e) {
                    e.printStackTrace();
                } finally {
                    httpResponse = null;
                }
            }
            if (httpClient != null) {
                try {
                    httpClient.close();
                } catch (IOException e) {
                    e.printStackTrace();
                } finally {
                    httpClient = null;
                }
            }
        }
    }

    private ExamScorePushInfo buildexamScorePushInfo(ExamScoreEntity examScore,
                                                     ExamRecordDataEntity examRecordDataEntity,
                                                     ExamStudentEntity examStudentEntity) {
        ExamScorePushInfo examScorePushInfo = new ExamScorePushInfo();
        examScorePushInfo.setExamId(examStudentEntity.getExamId());
        ExamSettingsCacheBean examBean = ExamCacheTransferHelper.getCachedExam(examStudentEntity.getExamId(),
                examStudentEntity.getStudentId(), examStudentEntity.getExamStageId());

        examScorePushInfo.setExamName(examBean.getName());
        examScorePushInfo.setStudentName(examStudentEntity.getStudentName());
        examScorePushInfo.setStudentCode(examStudentEntity.getStudentCode());
        examScorePushInfo.setIdentityNumber(examStudentEntity.getIdentityNumber());

        CourseCacheBean courseBean = ExamCacheTransferHelper.getCachedCourse(examStudentEntity.getCourseId());

        examScorePushInfo.setCourseName(courseBean.getName());
        examScorePushInfo.setCourseCode(examStudentEntity.getCourseCode());
        examScorePushInfo.setStartTime(examRecordDataEntity.getStartTime());
        examScorePushInfo.setEndTime(examRecordDataEntity.getEndTime());
        examScorePushInfo.setSuccPercent(examRecordDataEntity.getFaceSuccessPercent());
        examScorePushInfo.setIsIllegality(examRecordDataEntity.getIsIllegality());
        examScorePushInfo.setScoreId(examScore.getId());
        examScorePushInfo.setTotalScore(examScore.getTotalScore());
        examScorePushInfo.setObjectiveScore(examScore.getObjectiveScore());
        examScorePushInfo.setSubjectiveScore(examScore.getSubjectiveScore());
        return examScorePushInfo;
    }

}
