package cn.com.qmth.examcloud.core.oe.admin.stater;

import cn.com.qmth.examcloud.core.oe.admin.dao.UniqueRuleHolder;
import cn.com.qmth.examcloud.support.filestorage.FileStorageUtil;
import cn.com.qmth.examcloud.web.jpa.DataIntegrityViolationTransverter;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.domain.EntityScan;
import org.springframework.boot.autoconfigure.web.servlet.MultipartAutoConfiguration;
import org.springframework.cloud.client.discovery.EnableDiscoveryClient;
import org.springframework.cloud.netflix.eureka.EnableEurekaClient;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.jpa.repository.config.EnableJpaAuditing;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;
import org.springframework.data.mongodb.repository.config.EnableMongoRepositories;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.transaction.annotation.EnableTransactionManagement;
import org.springframework.web.multipart.MultipartResolver;
import org.springframework.web.multipart.commons.CommonsMultipartResolver;

@EnableAsync
@Configuration
@EnableJpaAuditing
@EnableTransactionManagement
@EnableEurekaClient
@EnableDiscoveryClient
@ComponentScan(basePackages = {"cn.com.qmth"})
@EntityScan(basePackages = {"cn.com.qmth"})
@EnableJpaRepositories(basePackages = {"cn.com.qmth"})
@EnableMongoRepositories(basePackages = {"cn.com.qmth"})
@SpringBootApplication(exclude = {MultipartAutoConfiguration.class})
public class OEAdminApp {

    static {
        if (System.getProperty("log.commonLevel") == null) {
            System.setProperty("log.commonLevel", "INFO");
        }
        System.setProperty("hibernate.dialect.storage_engine", "innodb");

        DataIntegrityViolationTransverter.setUniqueRules(UniqueRuleHolder.getUniqueRuleList());
    }

    public static void main(String[] args) {
        // AppBootstrap.run(OEAdminApp.class, args);
        SpringApplication.run(OEAdminApp.class, args);

        FileStorageUtil.initYunSite();
        FileStorageUtil.initYunClient();
    }

    @Bean(name = "multipartResolver")
    public MultipartResolver multipartResolver() {
        CommonsMultipartResolver resolver = new CommonsMultipartResolver();
        resolver.setDefaultEncoding("UTF-8");
        resolver.setResolveLazily(true);
        resolver.setMaxInMemorySize(2);
        resolver.setMaxUploadSize(200 * 1024 * 1024);
        return resolver;
    }

}
